import pytest
from scout.build import build_institute

def test_build_institute(parsed_institute):
    ins = build_institute(
        internal_id = parsed_institute['institute_id'],
        display_name = parsed_institute['display_name'],
        sanger_recipients = parsed_institute['sanger_recipients'],
    )
    
    assert ins['internal_id'] == ins['_id'] == parsed_institute['institute_id']
    assert isinstance(ins['sanger_recipients'], list)

def test_build_institute_no_sanger():
    ## GIVEN a institute without sanger recipients
    institute_info = dict(
            internal_id = 'cust000',
            display_name = 'test'
    )
    ## WHEN building the institute
    ins = build_institute(
        internal_id = institute_info['internal_id'],
        display_name = institute_info['display_name']
    )
    
    assert 'sanger_recipients' not in ins




# -*- coding: utf-8 -*-
from flask import url_for


# def test_panels(client, real_database, user_info):
#     # GIVEN a user connected to one institute in the database
#     institute_obj = real_database.institute(user_info['institutes'][0])
#     # WHEN accessing the panels overview page
#     resp = client.get(url_for('panels.panels'))
#     # THEN it should list the panels for that institute
#     assert resp.status_code == 200
#     assert institute_obj['display_name'] in resp.data

#     for panel_obj in real_database.panels(institute_id=institute_obj['_id']):
#         assert panel_obj['display_name'] in resp.data

def test_panel(client, real_database, panel_info):
    # GIVEN a panel in the database
    panel_obj = real_database.gene_panels()[0]
    # WHEN accessing the panel view
    resp = client.get(url_for('panels.panel', panel_id=panel_obj['_id']))
    # THEN it should display the panel with all the genes
    assert resp.status_code == 200
    assert panel_obj['panel_name'].encode() in resp.data
    # assert panel_obj['version'] in resp.data
    assert resp.data.count('href="/genes/'.encode()) == len(panel_obj['genes'])



# -*- coding: utf-8 -*-
SECRET_KEY = 'this is not secret...'
REMEMBER_COOKIE_NAME = 'scout_remember_me'

MONGO_DBNAME = 'scoutTest'

BOOTSTRAP_SERVE_LOCAL = True
TEMPLATES_AUTO_RELOAD = True

DEBUG_TB_INTERCEPT_REDIRECTS = False

# Flask-mail: http://pythonhosted.org/flask-mail/
# see: https://bitbucket.org/danjac/flask-mail/issue/3
MAIL_SERVER = 'smtp.gmail.com'
MAIL_PORT = 587
MAIL_USE_TLS = True
MAIL_USE_SSL = False

# Chanjo-Report
REPORT_LANGUAGE = 'en'
ACCEPT_LANGUAGES = ['en', 'sv']

# FEATURE FLAGS
SHOW_CAUSATIVES = False



import logging

from datetime import datetime

from scout.exceptions import (PedigreeError, ConfigError, IntegrityError)

from . import build_individual

log = logging.getLogger(__name__)

def build_phenotype(phenotype_id, adapter):
    """Build a small phenotype object

        Build a dictionary with phenotype_id and description

    Args:
        phenotype_id (str): The phenotype id
        adapter (scout.adapter.MongoAdapter)

    Returns:
        phenotype_obj (dict):

        dict(
            phenotype_id = str,
            feature = str, # description of phenotype
            )
    """
    phenotype_obj = {}
    phenotype = adapter.hpo_term(phenotype_id)
    if phenotype:
        phenotype_obj['phenotype_id'] = phenotype['hpo_id']
        phenotype_obj['feature'] = phenotype['description']
    return phenotype

def build_case(case_data, adapter):
    """Build a case object that is to be inserted to the database

    Args:
        case_data (dict): A dictionary with the relevant case information
        adapter (scout.adapter.MongoAdapter)

    Returns:
        case_obj (dict): A case object

    dict(
        case_id = str, # required=True, unique
        display_name = str, # If not display name use case_id
        owner = str, # required

        # These are the names of all the collaborators that are allowed to view the
        # case, including the owner
        collaborators = list, # List of institute_ids
        assignee = str, # _id of a user
        individuals = list, # list of dictionaries with individuals
        created_at = datetime,
        updated_at = datetime,
        suspects = list, # List of variants referred by there _id
        causatives = list, # List of variants referred by there _id

        synopsis = str, # The synopsis is a text blob
        status = str, # default='inactive', choices=STATUS
        is_research = bool, # default=False
        research_requested = bool, # default=False
        rerun_requested = bool, # default=False

        analysis_date = datetime,
        analyses = list, # list of dict

        # default_panels specifies which panels that should be shown when
        # the case is opened
        panels = list, # list of dictionaries with panel information

        dynamic_gene_list = list, # List of genes

        genome_build = str, # This should be 37 or 38
        genome_version = float, # What version of the build

        rank_model_version = str,
        rank_score_threshold = int, # default=8

        phenotype_terms = list, # List of dictionaries with phenotype information
        phenotype_groups = list, # List of dictionaries with phenotype information

        madeline_info = str, # madeline info is a full xml file

        multiqc = str, # path to dir with multiqc information

        vcf_files = dict, # A dictionary with vcf files

        diagnosis_phenotypes = list, # List of references to diseases
        diagnosis_genes = list, # List of references to genes

        has_svvariants = bool, # default=False

        is_migrated = bool # default=False

    )
    """
    log.info("build case with id: {0}".format(case_data['case_id']))
    case_obj = {
        '_id': case_data['case_id'],
        'display_name': case_data.get('display_name', case_data['case_id']),
    }

    # Check if institute exists in database
    try:
        institute_id = case_data['owner']
    except KeyError as err:
        raise ConfigError("Case has to have a institute")
    institute_obj = adapter.institute(institute_id)
    if not institute_obj:
        raise IntegrityError("Institute %s not found in database" % institute_id)
    case_obj['owner'] = case_data['owner']

    # Owner allways has to be part of collaborators
    collaborators = set(case_data.get('collaborators', []))
    collaborators.add(case_data['owner'])
    case_obj['collaborators'] = list(collaborators)

    if case_data.get('assignee'):
        case_obj['assignees'] = [case_data['assignee']]

    # Individuals
    ind_objs = []
    try:
        for individual in case_data.get('individuals', []):
            ind_objs.append(build_individual(individual))
    except Exception as error:
        ## TODO add some action here
        raise error
    # sort the samples to put the affected individual first
    sorted_inds = sorted(ind_objs, key=lambda ind: -ind['phenotype'])
    case_obj['individuals'] = sorted_inds

    now = datetime.now()
    case_obj['created_at'] = now
    case_obj['updated_at'] = now

    if case_data.get('suspects'):
        case_obj['suspects'] = case_data['suspects']
    if case_data.get('causatives'):
        case_obj['causatives'] = case_data['causatives']

    case_obj['synopsis'] = case_data.get('synopsis', '')

    case_obj['status'] = 'inactive'
    case_obj['is_research'] = False
    case_obj['research_requested'] = False
    case_obj['rerun_requested'] = False

    analysis_date = case_data.get('analysis_date')
    if analysis_date:
        case_obj['analysis_date'] = analysis_date

    # We store some metadata and references about gene panels in 'panels'
    case_panels = case_data.get('gene_panels', [])
    default_panels = case_data.get('default_panels', [])
    panels = []

    for panel_name in case_panels:
        panel_obj = adapter.gene_panel(panel_name)
        if not panel_obj:
            raise IntegrityError("Panel %s does not exist in database" % panel_name)
        panel = {
            'panel_id': panel_obj['_id'],
            'panel_name': panel_obj['panel_name'],
            'display_name': panel_obj['display_name'],
            'version': panel_obj['version'],
            'updated_at': panel_obj['date'],
            'nr_genes': len(panel_obj['genes'])
        }
        if panel_name in default_panels:
            panel['is_default'] = True
        else:
            panel['is_default'] = False
        panels.append(panel)

    case_obj['panels'] = panels

    case_obj['dynamic_gene_list'] = {}

    # Meta data
    genome_build = case_data.get('genome_build', '37')
    if not genome_build in ['37', '38']:
        pass
        ##TODO raise exception if invalid genome build was used

    case_obj['genome_build'] = genome_build
    case_obj['genome_version'] = case_data.get('genome_version')

    if case_data.get('rank_model_version'):
        case_obj['rank_model_version'] = str(case_data['rank_model_version'])

    if case_data.get('rank_score_threshold'):
        case_obj['rank_score_threshold'] = float(case_data['rank_score_threshold'])

    # phenotype information
    phenotypes = []
    for phenotype in case_data.get('phenotype_terms', []):
        phenotype_obj = build_phenotype(phenotype, adapter)
        if phenotype_obj:
            phenotypes.append(phenotype_obj)
    if phenotypes:
        case_obj['phenotype_terms'] = phenotypes

    # phenotype groups
    phenotype_groups = []
    for phenotype in case_data.get('phenotype_groups', []):
        phenotype_obj = build_phenotype(phenotype, adapter)
        if phenotype_obj:
            phenotype_groups.append(phenotype_obj)
    if phenotype_groups:
        case_obj['phenotype_groups'] = phenotype_groups


    # Files
    case_obj['madeline_info'] = case_data.get('madeline_info')
    if 'multiqc' in case_data:
        case_obj['multiqc'] = case_data.get('multiqc')
    case_obj['vcf_files'] = case_data.get('vcf_files', {})
    case_obj['delivery_report'] = case_data.get('delivery_report')

    case_obj['has_svvariants'] = False
    if (case_obj['vcf_files'].get('vcf_sv') or case_obj['vcf_files'].get('vcf_sv_research')):
        case_obj['has_svvariants'] = True

    case_obj['has_strvariants'] = False
    if (case_obj['vcf_files'].get('vcf_str')):
        case_obj['has_strvariants'] = True

    case_obj['is_migrated'] = False

    return case_obj



from scout.parse.variant.coordinates import (get_cytoband_coordinates, get_sub_category, 
                                             get_length, get_end, parse_coordinates)


class CyvcfVariant(object):
    """Mock a cyvcf variant
    
    Default is to return a variant with three individuals high genotype 
    quality.
    """
    def __init__(self, chrom='1', pos=80000, ref='A', alt='C', end=None, 
                 gt_quals=[60, 60, 60], gt_types=[1, 1, 0], var_type='snv',
                 info_dict={}):
        super(CyvcfVariant, self).__init__()
        self.CHROM = chrom
        self.POS = pos
        self.REF = ref
        self.ALT = [alt]
        self.end = end or pos
        self.gt_quals = gt_quals
        self.gt_types = gt_types
        self.var_type = var_type
        self.INFO = info_dict


def test_parse_coordinates_snv():
    variant = CyvcfVariant()
    
    coordinates = parse_coordinates(variant, 'snv')
    
    assert coordinates['position'] == variant.POS

def test_parse_coordinates_indel():
    variant = CyvcfVariant(alt='ACCC', end=80003)
    
    coordinates = parse_coordinates(variant, 'snv')
    
    assert coordinates['position'] == variant.POS
    assert coordinates['end'] == variant.end

def test_parse_coordinates_translocation():
    info_dict = {
        'SVTYPE': 'BND',
    }
    variant = CyvcfVariant(
        ref='N', 
        alt='N[hs37d5:12060532[',
        pos=724779,
        end=724779,
        var_type='sv',
        info_dict=info_dict,
    )
    
    coordinates = parse_coordinates(variant, 'sv')
    
    assert coordinates['position'] == variant.POS
    assert coordinates['end'] == 12060532
    assert coordinates['end_chrom'] == 'hs37d5'
    assert coordinates['length'] == 10e10
    assert coordinates['sub_category'] ==  'bnd'

def test_parse_coordinates_translocation_2():
    info_dict = {
        'SVTYPE': 'BND',
    }
    variant = CyvcfVariant(
        ref='N', 
        alt='N[GL000232.1:25141[',
        pos=724779,
        end=724779,
        var_type='sv',
        info_dict=info_dict,
    )
    
    coordinates = parse_coordinates(variant, 'sv')
    
    assert coordinates['position'] == variant.POS
    assert coordinates['end'] == 25141
    assert coordinates['end_chrom'] == 'GL000232.1'
    assert coordinates['length'] == 10e10
    assert coordinates['sub_category'] ==  'bnd'
    
    
###### parse subcategory #######
def test_get_subcategory_snv():
    alt_len = 1
    ref_len = 1
    category = 'snv'
    svtype = None
    
    sub_category = get_sub_category(alt_len, ref_len, category, svtype)
    
    assert sub_category == 'snv'

def test_get_subcategory_indel():
    alt_len = 1
    ref_len = 3
    category = 'snv'
    svtype = None
    
    sub_category = get_sub_category(alt_len, ref_len, category, svtype)
    
    assert sub_category == 'indel'

###### parse length #######

# get_length(alt_len, ref_len, category, pos, end, svtype=None, svlen=None)
def test_get_length_snv():
    alt_len = 1
    ref_len = 1
    category = 'snv'
    pos = end = 879537
    
    length = get_length(alt_len, ref_len, category, pos, end)
    
    assert length == 1

def test_get_length_indel():
    alt_len = 3
    ref_len = 1
    category = 'snv'
    pos = end = 879537
    
    length = get_length(alt_len, ref_len, category, pos, end)
    
    assert length == 2

def test_get_sv_length_small_ins():
    ## GIVEN an insertion with whole sequence in alt field
    alt_len = 296
    ref_len = 1
    category = 'sv'
    # Pos and end is same for insertions
    pos = end = 144343218
    svtype = 'ins'
    svlen = 296
    
    ## WHEN parsing the length
    length = get_length(alt_len, ref_len, category, pos, end, svtype, svlen)
    
    ## THEN assert that the length is correct
    assert length == 296

def test_get_sv_length_large_ins_no_length():
    ## GIVEN an imprecise insertion
    alt_len = 5
    ref_len = 1
    category = 'sv'
    # Pos and end is same for insertions
    pos = end = 133920667
    svtype = 'ins'
    svlen = None
    
    ## WHEN parsing the length
    length = get_length(alt_len, ref_len, category, pos, end, svtype, svlen)
    
    ## THEN assert that the length is correct
    assert length == -1

def test_get_sv_length_translocation():
    ## GIVEN an translocation
    alt_len = 16
    ref_len = 1
    category = 'sv'
    pos = 726044
    end = None
    svtype = 'bnd'
    svlen = None
    
    ## WHEN parsing the length
    length = get_length(alt_len, ref_len, category, pos, end, svtype, svlen)
    
    ## THEN assert that the length is correct
    assert length == 10e10

def test_get_sv_length_cnvnator_del():
    ## GIVEN an cnvnator type deletion
    alt_len = 5
    ref_len = 1
    category = 'sv'
    pos = 1
    end = 10000
    svtype = 'del'
    svlen = -10000
    
    ## WHEN parsing the length
    length = get_length(alt_len, ref_len, category, pos, end, svtype, svlen)
    
    ## THEN assert that the length is correct
    assert length == 10000

def test_get_sv_length_del_no_length():
    ## GIVEN an deletion without len
    alt_len = 5
    ref_len = 1
    category = 'sv'
    pos = 869314
    end = 870246
    svtype = 'del'
    svlen = None
    
    ## WHEN parsing the length
    length = get_length(alt_len, ref_len, category, pos, end, svtype, svlen)
    
    ## THEN assert that the length is correct
    assert length == end - pos

###### parse end #######
# get_end(pos, alt, category, snvend, svend, svlen)

# snv/indels are easy since cyvcf2 are parsing the end for us

def test_get_end_snv():
    alt = 'T'
    category = 'snv'
    pos = snvend = 879537
    
    end = get_end(pos, alt, category, snvend, svend=None, svlen=None)
    
    assert end == snvend

def test_get_end_indel():
    alt = 'C'
    category = 'indel'
    pos = 302253
    snvend = 302265
    
    end = get_end(pos, alt, category, snvend, svend=None, svlen=None)
    
    assert end == snvend

# SVs are much harder since there are a lot of corner cases
# Most SVs (except translocations) have END annotated in INFO field
# The problem is that many times END==POS and then we have to do some magic on our own

def test_get_end_tiddit_translocation():
    ## GIVEN a translocation
    alt = 'N[hs37d5:12060532['
    category = 'sv'
    pos = 724779
    
    ## WHEN parsing the end coordinate
    end = get_end(pos, alt, category, snvend=None, svend=None, svlen=None)
    
    ## THEN assert that the end is the same as en coordinate described in alt field
    assert end == 12060532

def test_get_end_tiddit_translocation():
    ## GIVEN a translocation
    alt = 'N[hs37d5:12060532['
    category = 'sv'
    pos = 724779
    
    ## WHEN parsing the end coordinate
    end = get_end(pos, alt, category, snvend=None, svend=None, svlen=None)
    
    ## THEN assert that the end is the same as en coordinate described in alt field
    assert end == 12060532

def test_get_end_deletion():
    ## GIVEN a translocation
    alt = '<DEL>'
    category = 'sv'
    pos = 869314
    svend = 870246
    svlen = None
    
    ## WHEN parsing the end coordinate
    end = get_end(pos, alt, category, snvend=None, svend=svend, svlen=svlen)
    
    ## THEN assert that the end is the same as en coordinate described in alt field
    assert end == svend





from scout.parse.exac import (parse_exac_line, parse_exac_genes)

def test_parse_exac_line(exac_handle):
    """Test to parse a exac line"""
    header = next(exac_handle).rstrip().split('\t')
    first_gene = next(exac_handle)
    
    gene_info = parse_exac_line(header=header, line=first_gene)
    
    assert gene_info['hgnc_symbol'] == first_gene.split('\t')[1]

def test_parse_exac_genes(exac_handle):
    
    genes = parse_exac_genes(exac_handle)
    
    for gene in genes:
        assert gene['hgnc_symbol']


# -*- coding: utf-8 -*-
"""

"main concept of MongoDB is embed whenever possible"
Ref: http://stackoverflow.com/questions/4655610#comment5129510_4656431
"""
from __future__ import absolute_import, unicode_literals
from datetime import datetime

whitelist = dict(
    email = str, # required, unique
    created_at = datetime, 
    institutes = list, # list of institute_ids
)



import logging

LOG = logging.getLogger(__name__)

def get_dashboard_info(adapter, institute_id=None):
    """Returns cases with phenotype
    
        If phenotypes are provided search for only those
    
    Args:
        adapter(adapter.MongoAdapter)
        institute_id(str)
    
    Returns:
        data(dict): Dictionary with relevant information
    """
    general_info = get_general_case_info(adapter, institute_id)
    total_cases = general_info['total_cases']
    
    data = {'total_cases': total_cases}
    if total_cases == 0:
        return data
    
    data['pedigree'] = []
    for ped_info in general_info['pedigree'].values():
        ped_info['percent'] = ped_info['count'] / total_cases
        data['pedigree'].append(ped_info)
        
    
    data['cases'] = get_case_groups(adapter, total_cases, institute_id)
    
    data['analysis_types'] = get_analysis_types(adapter, total_cases, institute_id)

    # Fetch variant information
    LOG.info("Fetch sanger variants")
    validation_query = {'sanger_ordered': True}

    # Case level information
    validation_cases = set()
    validated_cases = set()
    
    # Variant level information
    validated_tp = set()
    validated_fp = set()

    LOG.info("Find all validated variants with query {}".format(institute_id))
    validation_ordered = adapter.variant_collection.find(validation_query)

    case_ids = general_info['case_ids']
    nr_ordered = 0
    for nr_ordered, variant in enumerate(validation_ordered,1):
        case_id = variant['case_id']
        if institute_id:
            if not case_id in case_ids:
                continue
        variant_id = variant['_id']
        # Add case id to validation cases
        validation_cases.add(case_id)

        validation = variant.get('validation')
        if validation == 'True positive':
            validated_tp.add(variant_id)
            validated_cases.add(case_id)

        elif validation == 'False positive':
            validated_fp.add(variant_id)
            validated_cases.add(case_id)
    
    nr_validation_cases = len(validation_cases)

    overview = [
        {
            'title': 'Phenotype terms',
            'count': general_info['phenotype_cases'],
            'percent': general_info['phenotype_cases'] / total_cases,
        }, 
        {
            'title': 'Causative variants',
            'count': general_info['causative_cases'],
            'percent': general_info['causative_cases'] / total_cases,
        }, 
        {
            'title': 'Pinned variants',
            'count': general_info['pinned_cases'],
            'percent': general_info['pinned_cases'] / total_cases,
        }, 
        {
            'title': 'Cohort tag',
            'count': general_info['cohort_cases'],
            'percent': general_info['cohort_cases'] / total_cases,
        }
    ]
    if nr_validation_cases:
        overview.append(
        {
            'title': 'Validation ordered',
            'count': nr_validation_cases,
            'percent': nr_validation_cases / total_cases,
        })
        overview.append(
        {
            'title': 'Validated',
            'count': len(validated_cases),
            'percent': len(validated_cases) / nr_validation_cases,
        })

    data['overview'] = overview
    
    variants = []
    nr_validated = len(validated_tp) + len(validated_fp)
    if nr_ordered:
        variants.append(
            {
                'title': 'Validation ordered',
                'count': nr_ordered,
                'percent': 1
            }
        )
    if nr_validated:
            variants.append(
                {
                    'title': 'Validated True Positive',
                    'count': len(validated_tp),
                    'percent': len(validated_tp) / nr_validated,
                }
            )
            
            variants.append(
                {
                    'title': 'Validated False Positive',
                    'count': len(validated_fp),
                    'percent': len(validated_fp) / nr_validated,
                }
            )
    
    data['variants'] = variants
    
    return data

def get_general_case_info(adapter, institute_id=None):
    """Return general information about cases
    
    Args:
        adapter(adapter.MongoAdapter)
        institute_id(str)
    
    Returns:
        general(dict)
    """
    general = {}
    # Fetch information about cases with certain activities
    cases = adapter.cases(collaborator=institute_id)
    
    phenotype_cases = 0
    causative_cases = 0
    pinned_cases = 0
    cohort_cases = 0
    
    pedigree = {
        1: {
            'title': 'Single',
            'count': 0
        },
        2: {
            'title': 'Duo',
            'count': 0
        },
        3: {
            'title': 'Trio',
            'count': 0
        },
        'many': {
            'title': 'Many',
            'count': 0
        },
    }
    
    case_ids = set()
    
    total_cases = 0
    for total_cases,case in enumerate(cases,1):
        # If only looking at one institute we need to save the case ids
        if institute_id:
            case_ids.add(case['_id'])
        if case.get('phenotype_terms'):
            phenotype_cases += 1
        if case.get('causatives'):
            causative_cases += 1
        if case.get('suspects'):
            pinned_cases += 1
        if case.get('cohorts'):
            cohort_cases += 1
        
        nr_individuals = len(case.get('individuals',[]))
        if nr_individuals == 0:
            continue
        if nr_individuals > 3:
            pedigree['many']['count'] += 1
        else:
            pedigree[nr_individuals]['count'] += 1
    
    general['total_cases'] = total_cases
    general['phenotype_cases'] = phenotype_cases
    general['causative_cases'] = causative_cases
    general['pinned_cases'] = pinned_cases
    general['cohort_cases'] = cohort_cases
    general['pedigree'] = pedigree
    general['case_ids'] = case_ids

    return general
    

def get_case_groups(adapter, total_cases, institute_id=None):
    """Return the information about case groups
    
    Args:
        store(adapter.MongoAdapter)
        total_cases(int): Total number of cases
    
    Returns:
        cases(dict):
    """
    # Create a group with all cases in the database
    cases = [{'status': 'all', 'count': total_cases, 'percent': 1}]
    # Group the cases based on their status
    pipeline = []
    group = {'$group' : {'_id': '$status', 'count': {'$sum': 1}}}
    query = {}
    if institute_id:
        query = {'$match': {'owner': institute_id}}
    
    if query:
        pipeline.append(query)

    pipeline.append(group)
    res = adapter.case_collection.aggregate(pipeline)
    
    for status_group in res:
        cases.append({'status': status_group['_id'],
                      'count': status_group['count'],
                      'percent': status_group['count'] / total_cases})
    
    return cases

def get_analysis_types(adapter, total_cases, institute_id=None):
    """Return information about case status"""
    # Group cases based on analysis type of the individuals
    query = {}
    if institute_id:
        query = {'$match': {'owner': institute_id}}
    
    pipeline = []
    if query:
        pipeline.append(query)
    
    pipeline.append({'$unwind': '$individuals'})
    pipeline.append({'$group': {'_id': '$individuals.analysis_type', 'count': {'$sum': 1}}})
    analysis_query = adapter.case_collection.aggregate(pipeline)
    analysis_types = [{'name': group['_id'], 'count': group['count']} for group in analysis_query]
    
    return analysis_types


import logging
import click

LOG = logging.getLogger(__name__)

from scout.constants import (SEX_MAP, PHENOTYPE_MAP)


@click.command('individuals', short_help='Display individuals')
@click.option('-i', '--institute',
              help='institute id of related cases'
              )
@click.option('--causatives',
              is_flag=True,
              help='Has causative variants'
              )
@click.option('-c', '--case-id')
@click.pass_context
def individuals(context, institute, causatives, case_id):
    """Show all individuals from all cases in the database"""
    LOG.info("Running scout view individuals")
    adapter = context.obj['adapter']
    individuals = []

    if case_id:
        case = adapter.case(case_id=case_id)
        if case:
            cases = [case]
        else:
            LOG.info("Could not find case %s", case_id)
            return
    else:
        cases = [case_obj for case_obj in
                 adapter.cases(
                     collaborator=institute,
                     has_causatives=causatives)]
        if len(cases) == 0:
            LOG.info("Could not find cases that match criteria")
            return
        individuals = (ind_obj for case_obj in cases for ind_obj in case_obj['individuals'])

    click.echo("#case_id\tind_id\tdisplay_name\tsex\tphenotype\tmother\tfather")

    for case in cases:
        for ind_obj in case['individuals']:
            ind_info = [
                case['_id'], ind_obj['individual_id'],
                ind_obj['display_name'], SEX_MAP[int(ind_obj['sex'])],
                PHENOTYPE_MAP[ind_obj['phenotype']], ind_obj['mother'],
                ind_obj['father']
            ]
            click.echo('\t'.join(ind_info))







import logging
from pprint import pprint as pp
import intervaltree

from scout.build.genes.exon import build_exon
from pymongo.errors import (DuplicateKeyError, BulkWriteError)

from scout.exceptions import IntegrityError

LOG = logging.getLogger(__name__)

class GeneHandler(object):

    def load_hgnc_gene(self, gene_obj):
        """Add a gene object with transcripts to the database

        Arguments:
            gene_obj(dict)

        """
        #LOG.debug("Loading gene %s, build %s into database" %
        #             (gene_obj['hgnc_symbol'], gene_obj['build']))
        res = self.hgnc_collection.insert_one(gene_obj)
        #LOG.debug("Gene saved")
        return res

    def load_hgnc_bulk(self, gene_objs):
        """Load a bulk of hgnc gene objects
        
        Raises IntegrityError if there are any write concerns

        Args:
            gene_objs(iterable(scout.models.hgnc_gene))

        Returns:
            result (pymongo.results.InsertManyResult)
        """

        LOG.info("Loading gene bulk with length %s", len(gene_objs))
        try:
            result = self.hgnc_collection.insert_many(gene_objs)
        except (DuplicateKeyError, BulkWriteError) as err:
            raise IntegrityError(err)

        return result
    
    def load_hgnc_transcript(self, transcript_obj):
        """Add a transcript object to the database

        Arguments:
            transcript_obj(dict)

        """
        res = self.transcript_collection.insert_one(transcript_obj)
        return res

    def load_transcript_bulk(self, transcript_objs):
        """Load a bulk of transcript objects to the database

        Arguments:
            transcript_objs(iterable(scout.models.hgnc_transcript))

        """
        LOG.info("Loading transcript bulk")
        try:
            result = self.transcript_collection.insert_many(transcript_objs)
        except (DuplicateKeyError, BulkWriteError) as err:
            raise IntegrityError(err)
        
        return result

    def load_exon(self, exon_obj):
        """Add a exon object to the database

        Arguments:
            exon_obj(dict)

        """
        res = self.exon_collection.insert_one(exon_obj)
        return res

    def load_exon_bulk(self, exon_objs):
        """Load a bulk of exon objects to the database

        Arguments:
            exon_objs(iterable(scout.models.hgnc_exon))

        """
        try:
            result = self.exon_collection.insert_many(transcript_objs)
        except (DuplicateKeyError, BulkWriteError) as err:
            raise IntegrityError(err)
        
        return result

    def hgnc_gene(self, hgnc_identifier, build='37'):
        """Fetch a hgnc gene

            Args:
                hgnc_identifier(int)

            Returns:
                gene_obj(HgncGene)
        """
        if not build in ['37', '38']:
            build = '37'
        query = {}
        try:
            # If the identifier is a integer we search for hgnc_id
            hgnc_identifier = int(hgnc_identifier)
            query['hgnc_id'] = hgnc_identifier
        except ValueError:
            # Else we seach for a hgnc_symbol
            query['hgnc_symbol'] = hgnc_identifier

        query['build'] = build
        LOG.debug("Fetching gene %s" % hgnc_identifier)
        gene_obj = self.hgnc_collection.find_one(query)
        if not gene_obj:
            return None
        
        # Add the transcripts:
        transcripts = []
        tx_objs = self.transcripts(build=build, hgnc_id=gene_obj['hgnc_id'])
        if tx_objs.count() > 0:
            for tx in tx_objs:
                transcripts.append(tx)
        gene_obj['transcripts'] = transcripts
        
        return gene_obj

    def hgnc_id(self, hgnc_symbol, build='37'):
        """Query the genes with a hgnc symbol and return the hgnc id

        Args:
            hgnc_symbol(str)
            build(str)

        Returns:
            hgnc_id(int)
        """
        #LOG.debug("Fetching gene %s", hgnc_symbol)
        query = {'hgnc_symbol':hgnc_symbol, 'build':build}
        projection = {'hgnc_id':1, '_id':0}
        res = self.hgnc_collection.find(query, projection)

        if res.count() > 0:
            return res[0]['hgnc_id']
        else:
            return None

    def hgnc_genes(self, hgnc_symbol, build='37', search=False):
        """Fetch all hgnc genes that match a hgnc symbol

            Check both hgnc_symbol and aliases

            Args:
                hgnc_symbol(str)
                build(str): The build in which to search
                search(bool): if partial searching should be used

            Returns:
                result()
        """
        LOG.debug("Fetching genes with symbol %s" % hgnc_symbol)
        if search:
            # first search for a full match
            full_query = self.hgnc_collection.find({
                '$or': [
                    {'aliases': hgnc_symbol},
                    {'hgnc_id': int(hgnc_symbol) if hgnc_symbol.isdigit() else None},
                ],
                'build': build
            })
            if full_query.count() != 0:
                return full_query

            return self.hgnc_collection.find({
                'aliases': {'$regex': hgnc_symbol, '$options': 'i'},
                'build': build
            })

        return self.hgnc_collection.find({'build': build, 'aliases': hgnc_symbol})

    def all_genes(self, build='37'):
        """Fetch all hgnc genes

            Returns:
                result()
        """
        LOG.info("Fetching all genes")
        return self.hgnc_collection.find({'build': build}).sort('chromosome', 1)

    def nr_genes(self, build=None):
        """Return the number of hgnc genes in collection

        If build is used, return the number of genes of a certain build

            Returns:
                result()
        """
        if build:
            LOG.info("Fetching all genes from build %s",  build)
        else:
            LOG.info("Fetching all genes")

        return self.hgnc_collection.find({'build':build}).count()

    def drop_genes(self, build=None):
        """Delete the genes collection"""
        if build:
            LOG.info("Dropping the hgnc_gene collection, build %s", build)
            self.hgnc_collection.delete_many({'build': build})
        else:
            LOG.info("Dropping the hgnc_gene collection")
            self.hgnc_collection.drop()

    def drop_transcripts(self, build=None):
        """Delete the transcripts collection"""
        if build:
            LOG.info("Dropping the transcripts collection, build %s", build)
            self.transcript_collection.delete_many({'build': build})
        else:
            LOG.info("Dropping the transcripts collection")
            self.transcript_collection.drop()

    def drop_exons(self, build=None):
        """Delete the exons collection"""
        if build:
            LOG.info("Dropping the exons collection, build %s", build)
            self.exon_collection.delete_many({'build': build})
        else:
            LOG.info("Dropping the exons collection")
            self.exon_collection.drop()

    def ensembl_transcripts(self, build='37'):
        """Return a dictionary with ensembl ids as keys and transcripts as value.

        Args:
            build(str)
        
        Returns:
            ensembl_transcripts(dict): {<enst_id>: transcripts_obj, ...}
        """
        ensembl_transcripts = {}
        LOG.info("Fetching all transcripts")
        for transcript_obj in self.transcript_collection.find({'build':build}):
            enst_id = transcript_obj['transcript_id']
            ensembl_transcripts[enst_id] = transcript_obj
        LOG.info("Ensembl transcripts fetched")

        return ensembl_transcripts

    def hgncid_to_gene(self, build='37', genes=None):
        """Return a dictionary with hgnc_id as key and gene_obj as value

        The result will have ONE entry for each gene in the database.
        (For a specific build)

        Args:
            build(str):
            genes(iterable(scout.models.HgncGene)):

        Returns:
            hgnc_dict(dict): {<hgnc_id(int)>: <gene(dict)>}

        """
        hgnc_dict = {}
        LOG.info("Building hgncid_to_gene")
        if not genes:
            genes = self.hgnc_collection.find({'build':build})

        for gene_obj in genes:
            hgnc_dict[gene_obj['hgnc_id']] = gene_obj

        return hgnc_dict

    def hgncsymbol_to_gene(self, build='37', genes=None):
        """Return a dictionary with hgnc_symbol as key and gene_obj as value

        The result will have ONE entry for each gene in the database.
        (For a specific build)

        Args:
            build(str)
            genes(iterable(scout.models.HgncGene)):

        Returns:
            hgnc_dict(dict): {<hgnc_symbol(str)>: <gene(dict)>}

        """
        hgnc_dict = {}
        LOG.info("Building hgncsymbol_to_gene")
        if not genes:
            genes = self.hgnc_collection.find({'build':build})

        for gene_obj in genes:
            hgnc_dict[gene_obj['hgnc_symbol']] = gene_obj
        LOG.info("All genes fetched")
        return hgnc_dict

    def gene_by_alias(self, symbol, build='37'):
        """Return a iterable with hgnc_genes.

        If the gene symbol is listed as primary the iterable will only have
        one result. If not the iterable will include all hgnc genes that have
        the symbol as an alias.

        Args:
            symbol(str)
            build(str)

        Returns:
            res(pymongo.Cursor(dict))
        """
        res = self.hgnc_collection.find({'hgnc_symbol': symbol, 'build':build})
        if res.count() == 0:
            res = self.hgnc_collection.find({'aliases': symbol, 'build':build})

        return res

    def genes_by_alias(self, build='37', genes=None):
        """Return a dictionary with hgnc symbols as keys and a list of hgnc ids
             as value.

        If a gene symbol is listed as primary the list of ids will only consist
        of that entry if not the gene can not be determined so the result is a list
        of hgnc_ids

        Args:
            build(str)
            genes(iterable(scout.models.HgncGene)):

        Returns:
            alias_genes(dict): {<hgnc_alias>: {'true': <hgnc_id>, 'ids': {<hgnc_id_1>, <hgnc_id_2>, ...}}}
        """
        LOG.info("Fetching all genes by alias")
        # Collect one entry for each alias symbol that exists
        alias_genes = {}
        # Loop over all genes
        if not genes:
            genes = self.hgnc_collection.find({'build':build})

        for gene in genes:
            # Collect the hgnc_id
            hgnc_id = gene['hgnc_id']
            # Collect the true symbol given by hgnc
            hgnc_symbol = gene['hgnc_symbol']
            # Loop aver all aliases
            for alias in gene['aliases']:
                true_id = None
                # If the alias is the same as hgnc symbol we know the true id
                if alias == hgnc_symbol:
                    true_id = hgnc_id
                # If the alias is already in the list we add the id
                if alias in alias_genes:
                    alias_genes[alias]['ids'].add(hgnc_id)
                    if true_id:
                        alias_genes[alias]['true'] = hgnc_id
                else:
                    alias_genes[alias] = {
                        'true': hgnc_id,
                        'ids': set([hgnc_id])
                    }

        return alias_genes

    def get_id_transcripts(self, hgnc_id, build='37'):
        """Return a set with identifier transcript(s)

        Choose all refseq transcripts with NM symbols, if none where found choose ONE with NR,
        if no NR choose ONE with XM. If there are no RefSeq transcripts identifiers choose the 
        longest ensembl transcript.

        Args:
            hgnc_id(int)
            build(str)

        Returns:
            identifier_transcripts(set)

        """
        transcripts = self.transcripts(build=build, hgnc_id=hgnc_id)

        identifier_transcripts = set()
        longest = None
        nr = []
        xm = []
        for tx in transcripts:
            enst_id = tx['transcript_id']
            # Should we not check if it is longest?
            if not longest:
                longest = enst_id
            refseq_id = tx.get('refseq_id')
            if not refseq_id:
                continue
            
            if 'NM' in refseq_id:
                identifier_transcripts.add(enst_id)
            elif 'NR' in refseq_id:
                nr.append(enst_id)
            elif 'XM' in refseq_id:
                xm.append(enst_id)
        
        if identifier_transcripts:
            return identifier_transcripts
        
        if nr:
            return set([nr[0]])

        if xm:
            return set([xm[0]])
        
        return set([longest])

    def transcripts_by_gene(self, build='37'):
        """Return a dictionary with hgnc_id as keys and a list of transcripts as value
        
        Args:
            build(str)
        
        Returns:
            hgnc_transcripts(dict)
                
        """
        hgnc_transcripts = {}
        LOG.info("Fetching all transcripts")
        for transcript in self.transcript_collection.find({'build':build}):
            hgnc_id = transcript['hgnc_id']
            if not hgnc_id in hgnc_transcripts:
                hgnc_transcripts[hgnc_id] = []
            
            hgnc_transcripts[hgnc_id].append(transcript)
        
        return hgnc_transcripts

    def id_transcripts_by_gene(self, build='37'):
        """Return a dictionary with hgnc_id as keys and a set of id transcripts as value
        
        Args:
            build(str)
        
        Returns:
            hgnc_id_transcripts(dict)
        """
        hgnc_id_transcripts = {}
        LOG.info("Fetching all id transcripts")
        for gene_obj in self.hgnc_collection.find({'build': build}):
            hgnc_id = gene_obj['hgnc_id']
            id_transcripts = self.get_id_transcripts(hgnc_id=hgnc_id, build=build)
            hgnc_id_transcripts[hgnc_id] = id_transcripts

        return hgnc_id_transcripts

    def ensembl_genes(self, build='37'):
        """Return a dictionary with ensembl ids as keys and gene objects as value.

        Args:
            build(str)
        
        Returns:
            genes(dict): {<ensg_id>: gene_obj, ...}
        """
        genes = {}
        
        LOG.info("Fetching all genes")
        for gene_obj in self.hgnc_collection.find({'build':build}):
            ensg_id = gene_obj['ensembl_id']
            hgnc_id = gene_obj['hgnc_id']
            
            genes[ensg_id] = gene_obj
        
        LOG.info("Ensembl genes fetched")

        return genes
    
    def transcripts(self, build='37', hgnc_id=None):
        """Return all transcripts.
        
            If a gene is specified return all transcripts for the gene
        
        Args:
            build(str)
            hgnc_id(int)
        
        Returns:
            iterable(transcript)
        """
        
        query = {'build': build}
        if hgnc_id:
            query['hgnc_id'] = hgnc_id
        
        return self.transcript_collection.find(query)
    

    def to_hgnc(self, hgnc_alias, build='37'):
        """Check if a hgnc symbol is an alias

            Return the correct hgnc symbol, if not existing return None

            Args:
                hgnc_alias(str)

            Returns:
                hgnc_symbol(str)
        """
        result = self.hgnc_genes(hgnc_symbol=hgnc_alias, build=build)
        if result:
            for gene in result:
                return gene['hgnc_symbol']
        else:
            return None

    def add_hgnc_id(self, genes):
        """Add the correct hgnc id to a set of genes with hgnc symbols

        Args:
            genes(list(dict)): A set of genes with hgnc symbols only

        """
        genes_by_alias = self.genes_by_alias()

        for gene in genes:
            id_info = genes_by_alias.get(gene['hgnc_symbol'])
            if not id_info:
                LOG.warning("Gene %s does not exist in scout", gene['hgnc_symbol'])
                continue
            gene['hgnc_id'] = id_info['true']
            if not id_info['true']:
                if len(id_info['ids']) > 1:
                    LOG.warning("Gene %s has ambiguous value, please choose one hgnc id in result", gene['hgnc_symbol'])
                gene['hgnc_id'] = ','.join([str(hgnc_id) for hgnc_id in id_info['ids']])

    def get_coding_intervals(self, build='37', genes=None):
        """Return a dictionary with chromosomes as keys and interval trees as values

        Each interval represents a coding region of overlapping genes.

        Args:
            build(str): The genome build
            genes(iterable(scout.models.HgncGene)):

        Returns:
            intervals(dict): A dictionary with chromosomes as keys and overlapping genomic intervals as values
        """
        intervals = {}
        if not genes:
            genes = self.all_genes(build=build)
        LOG.info("Building interval trees...")
        for i,hgnc_obj in enumerate(genes):
            chrom = hgnc_obj['chromosome']
            start = max((hgnc_obj['start'] - 5000), 1)
            end = hgnc_obj['end'] + 5000

            # If this is the first time a chromosome is seen we create a new
            # interval tree with current interval
            if chrom not in intervals:
                intervals[chrom] = intervaltree.IntervalTree()
                intervals[chrom].addi(start, end, i)
                continue

            res = intervals[chrom].search(start, end)

            # If the interval did not overlap any other intervals we insert it and continue
            if not res:
                intervals[chrom].addi(start, end, i)
                continue

            # Loop over the overlapping intervals
            for interval in res:
                # Update the positions to new max and mins
                if interval.begin < start:
                    start = interval.begin

                if interval.end > end:
                    end = interval.end

                # Delete the old interval
                intervals[chrom].remove(interval)

            # Add the new interval consisting och the overlapping ones
            intervals[chrom].addi(start, end, i)

        return intervals

    def load_exons(self, exons, genes=None, build='37'):
        """Create exon objects and insert them into the database
        
        Args:
            exons(iterable(dict))
        """
        genes = genes or self.ensembl_genes(build)
        for exon in exons:
            exon_obj = build_exon(exon, genes)
            if not exon_obj:
                continue
            
            res = self.exon_collection.insert_one(exon_obj)

    def exons(self, hgnc_id=None, transcript_id=None,  build=None):
        """Return all exons
        
        Args:
            hgnc_id(int)
            transcript_id(str)
            build(str)
        
        Returns:
            exons(iterable(dict))
        """
        query = {}
        if build:
            query['build'] = build
        if hgnc_id:
            query['hgnc_id'] = hgnc_id
        if transcript_id:
            query['transcript_id'] = transcript_id
        
        return self.exon_collection.find(query)



import pytest

from scout.commands import cli
from click.testing import CliRunner

#Sanity check for cli
def test_version():
    runner = CliRunner()
    result = runner.invoke(cli, ['--version'])
    assert result.exit_code == 0




def build_phenotype(phenotype_info):
    """Build a phenotype term object
    
        Args:
            phenotype(dict)
    
        Returns:
            phenotype_obj(dict)
    """
    phenotype_obj = dict(
        phenotype_id = phenotype_info['phenotype_id'],
        disease_models = phenotype_info['disease_models']
    )
    
    return phenotype_obj



import logging

from pprint import pprint as pp

from flask import (abort, Blueprint, current_app, redirect, render_template,
                   request, url_for, send_from_directory, jsonify, flash)
from flask_login import current_user

from scout.server.extensions import store

from .controllers import get_dashboard_info

blueprint = Blueprint('dashboard', __name__, template_folder='templates')

LOG = logging.getLogger(__name__)

@blueprint.route('/dashboard', methods=['GET', 'POST'])
def index():
    """Display the Scout dashboard."""
    accessible_institutes = None
    if not 'admin' in current_user.roles:
        accessible_institutes = current_user.institutes
        if not accessible_institutes:
            flash('Not allowed to see information - please visit the dashboard later!')
            return redirect(url_for('cases.index'))

    institutes = [inst for inst in store.institutes(accessible_institutes)]
    # Insert a entry that displays all institutes in the beginning of the array
    institutes.insert(0, {'_id': None, 'display_name': 'All institutes'})

    institute_id = None
    if request.method=='POST':
        institute_id = request.form.get('institute')

    LOG.info("Fetch all cases with institute: %s", institute_id)
    data = get_dashboard_info(store, institute_id)
    data['institutes'] = institutes
    data['choice'] = institute_id
    total_cases = data['total_cases']
    
    LOG.info("Found %s cases", total_cases)
    if total_cases == 0:
        flash('no cases loaded for institute {} - please visit the dashboard later!'.format(institute_id), 'info')
        return redirect(url_for('cases.index'))
    
    return render_template(
        'dashboard/index.html', **data)



import logging
import json


from pprint import pprint as pp

from bson.json_util import dumps

import click

from scout.constants import CASE_STATUSES
from .utils import json_option

LOG = logging.getLogger(__name__)

@click.command('cases', short_help='Fetch cases')
@click.option('--case-id',
              help='Case id to search for'
)
@click.option('-i', '--institute',
              help='institute id of related cases'
)
@click.option('-r', '--reruns',
              is_flag=True,
              help='requested to be rerun'
)
@click.option('-f', '--finished',
              is_flag=True,
              help='archived or solved'
)
@click.option('--causatives',
              is_flag=True,
              help='Has causative variants'
)
@click.option('--research-requested',
              is_flag=True,
              help='If research is requested'
)
@click.option('--is-research',
              is_flag=True,
              help='If case is in research mode'
)
@click.option('-s', '--status',
              type=click.Choice(CASE_STATUSES),
              help='Specify what status to look for'
)
@json_option
@click.pass_context
def cases(context, case_id, institute, reruns, finished, causatives, research_requested,
          is_research, status, json):
    """Interact with cases existing in the database."""
    adapter = context.obj['adapter']

    models = []
    if case_id:
        case_obj = adapter.case(case_id=case_id)
        if case_obj:
            models.append(case_obj)

    else:
        models = adapter.cases(collaborator=institute, reruns=reruns,
                           finished=finished, has_causatives=causatives,
                           research_requested=research_requested,
                           is_research=is_research, status=status)
    if models.count():
        LOG.info("No cases could be found")

    if json:
        click.echo(dumps(models))
        return

    for model in models:
        pp(model)
        
    




from scout.constants import PAR_COORDINATES
from scout.utils.coordinates import is_par

def test_par_coordinates():
    ## GIVEN a position that is in the first par region
    chromosome = 'X'
    position = 60001
    build = '37'
        
    ## WHEN checking the coordinates for a variant
    res = PAR_COORDINATES[build][chromosome].search(position)
    
    ## THEN assert that there way a hit
    assert res
    
    ## GIVEN a position that is not in any par region
    chromosome = 'X'
    position = 26995210
    build = '37'
        
    ## WHEN checking the coordinates for a variant
    res = PAR_COORDINATES[build][chromosome].search(position)
    
    ## THEN assert that there way a hit
    assert not res

def test_is_par():
    ## GIVEN a position that is in the first par region
    chromosome = 'X'
    position = 60001
    build = '37'

    ## WHEN checking the coordinates for a variant
    
    ## THEN assert that there way a hit
    assert is_par(chromosome, position, build)
    
def test_is_par_wrong_chromosome():
    ## GIVEN a position that is in the first par region
    chromosome = '1'
    position = 60001
    build = '37'

    ## WHEN checking the coordinates for the positions

    ## THEN assert that there way a hit
    assert is_par(chromosome, position, build) is False



def get_hgnc_id(gene_info, adapter):
    """Get the hgnc id for a gene

        The proprity order will be
        1. if there is a hgnc id this one will be choosen
        2. if the hgnc symbol matches a genes proper hgnc symbol
        3. if the symbol ony matches aliases on several genes one will be
           choosen at random

        Args:
            gene_info(dict)
            adapter

        Returns:
            true_id(int)
    """
    hgnc_id = gene_info.get('hgnc_id')
    hgnc_symbol = gene_info.get('hgnc_symbol')

    true_id = None

    if hgnc_id:
        true_id = int(hgnc_id)
    else:
        gene_result = adapter.hgnc_genes(hgnc_symbol)
        if gene_result.count() == 0:
            raise Exception("No gene could be found for {}".format(hgnc_symbol))
        for gene in gene_result:
            if hgnc_symbol.upper() == gene.hgnc_symbol.upper():
                true_id = gene.hgnc_id
        if not gene_info['hgnc_id']:
            true_id = gene.hgnc_id

    return true_id



# -*- coding: utf-8 -*-
from functools import wraps

from flask import render_template, request, abort, flash
from flask_login import current_user


def templated(template=None):
    """Template decorator.

    Ref: http://flask.pocoo.org/docs/patterns/viewdecorators/
    """
    def decorator(f):
        @wraps(f)
        def decorated_function(*args, **kwargs):
            template_name = template
            if template_name is None:
                template_name = request.endpoint.replace('.', '/') + '.html'
            context = f(*args, **kwargs)
            if context is None:
                context = {}
            elif not isinstance(context, dict):
                return context
            return render_template(template_name, **context)
        return decorated_function
    return decorator


def public_endpoint(function):
    function.is_public = True
    return function


def institute_and_case(store, institute_id, case_name=None):
    """Fetch insitiute and case objects."""
    institute_obj = store.institute(institute_id)
    if institute_obj is None and institute_id != 'favicon.ico':
        flash("Can't find institute: {}".format(institute_id), 'warning')
        return abort(404)

    if case_name:
        if case_name:
            case_obj = store.case(institute_id=institute_id, display_name=case_name)
            if case_obj is None:
                return abort(404)

    # validate that user has access to the institute
    if not current_user.is_admin:
        if institute_id not in current_user.institutes:
            if not case_name or not any(inst_id in case_obj['collaborators'] for inst_id in
                                        current_user.institutes):
                # you don't have access!!
                flash("You don't have acccess to: {}".format(institute_id),'danger')
                return abort(403)

    # you have access!
    if case_name:
        return institute_obj, case_obj
    else:
        return institute_obj


def user_institutes(store, login_user):
    """Preprocess institute objects."""
    if login_user.is_admin:
        institutes = store.institutes()
    else:
        institutes = [store.institute(inst_id) for inst_id in login_user.institutes]

    return institutes



import datetime
from scout.parse.panel import (parse_gene_panel, parse_genes, parse_gene, get_panel_info)
from scout.utils.handle import get_file_handle

def test_parse_panel_info_no_info():
    ## GIVEN no information at all
    panel_lines = []
    ## WHEN parsing the information
    panel_info = get_panel_info(panel_lines)
    ## THEN assert that no information except a date is returned
    for key in panel_info:
        if key != 'date':
            assert panel_info[key] == None
        else:
            assert type(panel_info[key]) is type(datetime.datetime.now())

def test_parse_panel_info_lines():
    ## GIVEN a panel with some info
    panel_lines = [
        '##panel_id=panel1',
        '##institute=cust000',
        '##version=1.0',
        '##date=2016-12-09',
        '##display_name=Test panel',
        '#hgnc_id\thgnc_symbol',
        '7481\tMT-TF'
    ]
    ## WHEN parsing the information
    panel_info = get_panel_info(panel_lines)
    ## THEN assert that no information except a date is returned
    for key in panel_info:
        if key == 'panel_id':
            assert panel_info[key] == 'panel1'
        elif key == 'institute':
            assert panel_info[key] == 'cust000'
        elif key == 'version':
            assert panel_info[key] == '1.0'
        elif key == 'display_name':
            assert panel_info[key] == 'Test panel'
        elif key == 'date':
            assert type(panel_info[key]) is type(datetime.datetime.now())

def test_parse_panel_file(panel_handle):
    ## GIVEN a panel with some info
    panel_lines = panel_handle
    ## WHEN parsing the information
    panel_info = get_panel_info(panel_lines)
    ## THEN assert that no information except a date is returned
    for key in panel_info:
        if key == 'panel_id':
            assert panel_info[key] == 'panel1'
        elif key == 'institute':
            assert panel_info[key] == 'cust000'
        elif key == 'version':
            assert panel_info[key] == '1.0'
        elif key == 'display_name':
            assert panel_info[key] == 'Test panel'
        elif key == 'date':
            assert type(panel_info[key]) is type(datetime.datetime.now())


def test_parse_minimal_gene():
    ## GIVEN minimal gene line and header
    header = ["hgnc_id"]
    gene_line = ["10"]
    gene_info = dict(zip(header, gene_line))
    ## WHEN parsing the genes
    gene = parse_gene(gene_info)
    ## THEN assert that the gene is correctly parsed
    
    assert gene['hgnc_id'] == 10

def test_parse_gene():
    ## GIVEN gene line and header
    header = [
        "hgnc_id", 
        "hgnc_symbol",
        "disease_associated_transcripts",
        "reduced_penetrance",
        "genetic_disease_models",
        "mosaicism",
        "database_entry_version"
    ]
    hgnc_id = '10'
    hgnc_symbol = 'hello'
    transcripts = 'a,b,c'
    penetrance = 'YES'
    models = 'AR,AD'
    mosaicism = ''
    version = ''
    
    gene_line = [
        hgnc_id, 
        hgnc_symbol, 
        transcripts, 
        penetrance, 
        models,
        mosaicism,
        version
    ]
    gene_info = dict(zip(header, gene_line))
    
    ## WHEN parsing the genes
    gene = parse_gene(gene_info)
    ## THEN assert that the gene is correctly parsed
    assert gene['hgnc_id'] == int(hgnc_id)
    assert gene['hgnc_symbol'] == hgnc_symbol
    assert gene['transcripts'] == transcripts.split(',')
    assert gene['inheritance_models'] == models.split(',')
    assert gene['reduced_penetrance'] == True if penetrance else False
    assert gene['mosaicism'] == False
    assert gene['database_entry_version'] == version

def test_parse_panel_lines():
    ## GIVEN a iterable with panel lines
    panel_lines = [
        "##panel_id=panel1",
        "##institute=cust000",
        "##version=1.0",
        "##date=2016-12-09",
        "##display_name=Test panel",
        "#hgnc_id\thgnc_symbol\tdisease_associated_transcripts\treduced_penet"\
        "rance\tgenetic_disease_models\tmosaicism\tdatabase_entry_version\tor"\
        "iginal_hgnc",
        "7481\tMT-TF\t\t\t\t\t\tMT-TF\n"
    ]
    nr_genes = len([line for line in panel_lines if not line.startswith('#')])
    
    ## WHEN parsing the panel of genes
    genes = parse_genes(panel_lines)
    
    ## THEN assert that all genes from the file have been parsed
    assert len(genes) == nr_genes
    
    ## THEN assert that some genes exists in the panel
    for gene in genes:
        assert gene['hgnc_symbol']
        assert gene['hgnc_id']

def test_parse_panel_doublette():
    ## GIVEN a iterable with panel lines where one gene occurs twice
    panel_lines = [
        "#hgnc_id\thgnc_symbol\tdisease_associated_transcripts\treduced_penet"\
        "rance\tgenetic_disease_models\tmosaicism\tdatabase_entry_version\tor"\
        "iginal_hgnc",
        "7481\tMT-TF\t\t\t\t\t\tMT-TF\n"
        "7481\tMT-TF\t\t\t\t\t\tMT-TF\n"
    ]    
    ## WHEN parsing the panel of genes
    genes = parse_genes(panel_lines)
    
    ## THEN that the gene is only occuring once
    assert len(genes) == 1
    

def test_parse_panel_genes(panel_1_file):
    # GIVEN a gene panel file
    nr_genes = 0
    with open(panel_1_file, 'r') as f:
        for line in f:
            if not line.startswith('#'):
                nr_genes += 1

    # WHEN parsing the panel of genes
    f = get_file_handle(panel_1_file)
    genes = parse_genes(f)

    # THEN assert that all genes from the file have been parsed
    assert len(genes) == nr_genes

    # THEN assert that some genes exists in the panek
    for gene in genes:
        assert gene['hgnc_symbol']
        assert gene['hgnc_id']

def test_get_full_list_panel(panel_info):
    panel_1_file = panel_info['file']
    nr_genes = 0
    with open(panel_1_file, 'r') as f:
        for line in f:
            if not line.startswith('#'):
                nr_genes += 1

    panel = parse_gene_panel(
        path=panel_info['file'], 
        institute=panel_info['institute'], 
        panel_id=panel_info['panel_name'], 
        panel_type=panel_info['type'], 
        date=panel_info['date'], 
        version=panel_info['version'], 
        display_name=panel_info['full_name']
    )

    assert panel['panel_id'] == panel_info['panel_name']
    assert len(panel['genes']) == nr_genes
    assert panel['date'] == panel_info['date']
    assert panel['institute'] == panel_info['institute']

def test_parse_minimal_panel_lines_id():
    ## GIVEN a iterable with panel lines
    panel_lines = [
        "1"
    ]
    nr_genes = len([line for line in panel_lines if not line.startswith('#')])
    
    ## WHEN parsing the panel of genes
    genes = parse_genes(panel_lines)
    
    ## THEN assert that all genes from the file have been parsed
    assert len(genes) == nr_genes
    
    ## THEN assert that some genes exists in the panel
    for gene in genes:
        assert gene['hgnc_id'] == 1
        assert gene.get('hgnc_symbol') == None

def test_parse_minimal_panel_lines_symbol():
    ## GIVEN a iterable with panel lines
    panel_lines = [
        "ADK"
    ]
    nr_genes = len([line for line in panel_lines if not line.startswith('#')])
    
    ## WHEN parsing the panel of genes
    genes = parse_genes(panel_lines)
    
    ## THEN assert that all genes from the file have been parsed
    assert len(genes) == nr_genes
    
    ## THEN assert that some genes exists in the panel
    for gene in genes:
        assert gene.get('hgnc_id') == None
        assert gene.get('hgnc_symbol') == 'ADK'

def test_parse_panel_lines_excel_export():
    ## GIVEN a iterable with panel lines
    panel_lines = [
        "hgnc_id;hgnc_symbol",
        "13666;AAAS",
        "16262;YAP1"
    ]
    nr_genes = 2
    
    ## WHEN parsing the panel of genes
    genes = parse_genes(panel_lines)
    
    ## THEN assert that all genes from the file have been parsed
    assert len(genes) == nr_genes
    
    ## THEN assert that some genes exists in the panel
    gene1 = genes[0]
    assert gene1.get('hgnc_id') == 13666
    assert gene1.get('hgnc_symbol') == 'AAAS'

    gene2 = genes[1]
    assert gene2.get('hgnc_id') == 16262
    assert gene2.get('hgnc_symbol') == 'YAP1'

def test_parse_panel_lines_excel_export_empty_line():
    ## GIVEN a iterable with panel lines
    panel_lines = [
        "hgnc_id;hgnc_symbol",
        "13666;AAAS",
        "16262;YAP1",
        ";",
        "",
    ]
    nr_genes = 2
    
    ## WHEN parsing the panel of genes
    genes = parse_genes(panel_lines)
    
    ## THEN assert that all genes from the file have been parsed
    assert len(genes) == nr_genes

def test_parse_panel_lines_modified_excel_export():
    ## GIVEN a iterable with panel lines
    panel_lines = [
        "HGNC_IDnumber;HGNC_symbol",
        "13666;AAAS"
    ]
    nr_genes = 1
    
    ## WHEN parsing the panel of genes
    genes = parse_genes(panel_lines)
    
    ## THEN assert that all genes from the file have been parsed
    assert len(genes) == nr_genes
    
    ## THEN assert that some genes exists in the panel
    for gene in genes:
        assert gene.get('hgnc_id') == 13666
        assert gene.get('hgnc_symbol') == 'AAAS'



# -*- coding: utf-8 -*-
import pytest
import logging
import datetime

from pprint import pprint as pp

from cyvcf2 import VCF
import yaml
import pymongo


from pandas import DataFrame
# Adapter stuff
from mongomock import MongoClient
from scout.adapter.mongo import MongoAdapter as PymongoAdapter

from scout.utils.handle import get_file_handle

from scout.parse.case import parse_case
from scout.parse.panel import parse_gene_panel
from scout.parse.variant import parse_variant
from scout.parse.variant.headers import parse_rank_results_header
from scout.parse.hgnc import parse_hgnc_genes
from scout.parse.ensembl import (parse_ensembl_transcripts, parse_transcripts)
from scout.parse.exac import parse_exac_genes
from scout.parse.hpo import (parse_hpo_phenotypes, parse_hpo_genes, parse_hpo_diseases)

from scout.utils.link import link_genes
from scout.log import init_log
from scout.build import (build_institute, build_case, build_panel)
from scout.build.variant import build_variant
from scout.build.genes.hgnc_gene import build_hgnc_gene
from scout.build.genes.transcript import build_transcript
from scout.build.user import build_user
from scout.load import (load_hgnc_genes)
from scout.load.hpo import load_hpo
from scout.load.transcript import load_transcripts

# These are the reduced data files
from scout.demo.resources import (hgnc_reduced_path, transcripts37_reduced_path, genes37_reduced_path,
exac_reduced_path, hpogenes_reduced_path, hpoterms_reduced_path, hpo_to_genes_reduced_path,
hpo_phenotype_to_terms_reduced_path, mim2gene_reduced_path, genemap2_reduced_path,
transcripts38_reduced_path, genes38_reduced_path,)

from scout.demo import (research_snv_path, research_sv_path, clinical_snv_path,
                        clinical_sv_path, ped_path, load_path, panel_path)

from scout.models.hgnc_map import HgncGene

DATABASE = 'testdb'
REAL_DATABASE = 'realtestdb'

root_logger = logging.getLogger()
init_log(root_logger, loglevel='INFO')
logger = logging.getLogger(__name__)


##################### Gene fixtures #####################

@pytest.fixture
def transcript_info(request):
    transcript = dict(
        chrom = '1',
        ens_gene_id = 'ENSG00000176022',
        ens_transcript_id = 'ENST00000379198',
        start = 1167629,
        end = 1170421,
        refseq_mrna = 'NM_080605',
        refseq_mrna_pred = '',
        refseq_ncrna = '',
    )

    return transcript


@pytest.fixture
def test_gene(request):
    gene = {
        # This is the hgnc id, required:
        'hgnc_id': 1,
        # The primary symbol, required
        'hgnc_symbol': 'test',
        'ensembl_id': 'ensembl1',  # required
        'build': '37',  # '37' or '38', defaults to '37', required

        'chromosome': 1,  # required
        'start': 10,  # required
        'end': 100,  # required

        'description': 'A gene',  # Gene description
        'aliases': ['test'],  # Gene symbol aliases, includes hgnc_symbol, str
        'entrez_id': 1,
        'omim_id': 1,
        'pli_score': 1.0,
        'primary_transcripts': ['NM1'],  # List of refseq transcripts (str)
        'ucsc_id': '1',
        'uniprot_ids': ['1'],  # List of str
        'vega_id': '1',
    }
    return gene

@pytest.fixture
def parsed_gene(request):
    gene_info = {
        'hgnc_id': 1,
        'hgnc_symbol': 'AAA',
        'ensembl_id': 'ENSG1',
        'chrom': '1',
        'start': 10,
        'end': 100,
        'build': '37'
    }
    return gene_info


@pytest.fixture
def genes(request, genes37_handle, hgnc_handle, exac_handle,
          mim2gene_handle, genemap_handle, hpo_genes_handle):
    """Get a dictionary with the linked genes"""
    print('')

    gene_dict = link_genes(
        ensembl_lines=genes37_handle,
        hgnc_lines=hgnc_handle,
        exac_lines=exac_handle,
        mim2gene_lines=mim2gene_handle,
        genemap_lines=genemap_handle,
        hpo_lines=hpo_genes_handle
    )

    return gene_dict

@pytest.fixture
def ensembl_genes(request, gene_bulk):
    """Return a dictionary that maps ensembl ids on genes"""
    _ensembl_genes = {}
    for gene_obj in gene_bulk:
        _ensembl_genes[gene_obj['ensembl_id']] = gene_obj
    return _ensembl_genes

@pytest.fixture
def gene_bulk(genes):
    """Return a list with HgncGene objects"""
    bulk = []
    for gene_key in genes:
        bulk.append(build_hgnc_gene(genes[gene_key]))

    return bulk

@pytest.fixture
def transcript_objs(request, parsed_transcripts):
    """Return a list with transcript objs"""
    print('')

    _transcripts = []
    for tx_id in parsed_transcripts:
        tx_info = parsed_transcripts[tx_id]
        _transcripts.append(build_transcript(tx_info))

    return _transcripts

@pytest.fixture
def transcripts_df(request, transcripts):
    """Return a list with transcript objs"""
    print('')
    df_info = dict(
        chromosome_name = [],
        ensembl_gene_id = [],
        ensembl_transcript_id = [],
        transcript_start = [],
        transcript_end = [],
        refseq_mrna = [],
        refseq_mrna_predicted = [],
        refseq_ncrna = [],
    )

    for tx_info in transcripts:
        df_info['chromosome_name'].append(tx_info['chrom'])
        df_info['ensembl_gene_id'].append(tx_info['ensembl_gene_id'])
        df_info['ensembl_transcript_id'].append(tx_info['ensembl_transcript_id'])
        df_info['transcript_start'].append(tx_info['transcript_start'])
        df_info['transcript_end'].append(tx_info['transcript_end'])
        df_info['refseq_mrna'].append(tx_info.get('refseq_mrna', ''))
        df_info['refseq_mrna_predicted'].append(tx_info.get('refseq_mrna_predicted', ''))
        df_info['refseq_ncrna'].append(tx_info.get('refseq_ncrna', ''))

    df = DataFrame(df_info)

    return df


#############################################################
################# Hpo terms fixtures ########################
#############################################################
@pytest.fixture
def hpo_terms_handle(request, hpo_terms_file):
    """Get a file handle to a hpo terms file"""
    print('')
    hpo_lines = get_file_handle(hpo_terms_file)
    return hpo_lines


@pytest.fixture
def hpo_terms(request, hpo_terms_file):
    """Get a dictionary with the hpo terms"""
    print('')
    hpo_terms_handle = get_file_handle(hpo_terms_file)
    return parse_hpo_phenotypes(hpo_terms_handle)


@pytest.fixture
def hpo_disease_handle(request, hpo_disease_file):
    """Get a file handle to a hpo disease file"""
    print('')
    handle = get_file_handle(hpo_disease_file)
    return handle


@pytest.fixture
def hpo_diseases(request, hpo_disease_file):
    """Get a file handle to a hpo disease file"""
    print('')
    hpo_disease_handle = get_file_handle(hpo_disease_file)
    diseases = parse_hpo_diseases(hpo_disease_handle)
    return diseases


#############################################################
##################### Case fixtures #########################
#############################################################
@pytest.fixture(scope='function')
def ped_lines(request, scout_config):
    """Get the lines for a case"""
    case_lines = [
        "#Family ID	Individual ID	Paternal ID	Maternal ID	Sex	Phenotype",
        "643594	ADM1059A1	0	0	1	1",
        "643594	ADM1059A2	ADM1059A1	ADM1059A3	1	2",
        "643594	ADM1059A3	0	0	2	1",
    ]
    return case_lines


@pytest.fixture(scope='function')
def case_lines(request, scout_config):
    """Get the lines for a case"""
    case = parse_case(scout_config)
    return case


@pytest.fixture(scope='function')
def parsed_case(request, scout_config):
    """Get the lines for a case"""
    case = parse_case(scout_config)
    return case


@pytest.fixture(scope='function')
def case_obj(request, parsed_case):

    logger.info("Create a case obj")
    case = parsed_case
    case['_id'] = parsed_case['case_id']
    case['owner'] = parsed_case['owner']
    case['created_at'] = parsed_case['analysis_date']
    case['dynamic_gene_list'] = {}
    case['genome_version'] = None
    case['has_svvariants'] = True

    case['individuals'][0]['sex'] = '1'
    case['individuals'][1]['sex'] = '1'
    case['individuals'][2]['sex'] = '2'

    case['is_migrated'] = False
    case['is_research'] = False

    case['panels'] = [
        {
                'display_name': 'Test panel',
                'is_default': True,
                'nr_genes': 263,
                'panel_id': 'panel1',
                'panel_name': 'panel1',
                'updated_at': datetime.datetime(2018, 4, 25, 15, 43, 44, 823465),
                'version': 1.0
        }
    ]
    case['rerun_requested']= False
    case['research_requested'] = False
    case['status'] = 'inactive'
    case['synopsis'] = ''
    case['updated_at'] = parsed_case['analysis_date']

    return case


#############################################################
##################### Clinvar fixtures ######################
#############################################################
@pytest.fixture(scope='function')
def clinvar_variant(request):
    clivar_variant = {
        '_id' : '3eecfca5efea445eec6c19a53299043b',
        '##Local_ID' : '3eecfca5efea445eec6c19a53299043b',
        'Reference_allele' : 'C',
        'Alternate_allele' : 'A',
        'Chromosome' : '7',
        'Start' : '124491972',
        'Stop' : '124491972',
        'Clinical_significance' : 'Likely Pathogenic',
        'Condition_ID_value' : 'HP:0001298;HP:0002121',
        'clinvar_submission' : 'SUB666',
    }

    return clivar_variant


#############################################################
##################### Institute fixtures ####################
#############################################################
@pytest.fixture(scope='function')
def parsed_institute(request):
    print('')
    institute = {
        'institute_id': 'cust000',
        'display_name': 'test_institute',
        'sanger_recipients': ['john@doe.com', 'jane@doe.com']
    }

    return institute


@pytest.fixture(scope='function')
def institute_obj(request, parsed_institute):
    print('')
    logger.info('Building a institute')
    institute = build_institute(
        internal_id=parsed_institute['institute_id'],
        display_name=parsed_institute['display_name'],
        sanger_recipients=parsed_institute['sanger_recipients'],
    )
    return institute


#############################################################
##################### User fixtures #########################
#############################################################
@pytest.fixture(scope='function')
def parsed_user(request, institute_obj):
    """Return user info"""
    user_info = {
        'email': 'john@doe.com',
        'name': 'John Doe',
        'location': 'here',
        'institutes': [institute_obj['internal_id']],
        'roles': ['admin']
    }
    return user_info


@pytest.fixture(scope='function')
def user_obj(request, parsed_user):
    """Return a User object"""
    _user_obj = build_user(parsed_user)
    return _user_obj


#############################################################
##################### Adapter fixtures #####################
#############################################################

# We need to mokeypatch 'connect' function so the tests use a mongomock database
# @pytest.fixture(autouse=True)
# def no_connect(monkeypatch):
#     # from scout.adapter.client import get_connection
#     mongo = Mock(return_value=MongoClient())
#     print('hej')
#
#     monkeypatch.setattr('scout.adapter.client.get_connection', mongo)

@pytest.fixture(scope='function')
def database_name(request):
    """Get the name of the test database"""
    return DATABASE


@pytest.fixture(scope='function')
def pymongo_client(request):
    """Get a client to the mongo database"""

    logger.info("Get a mongomock client")
    start_time = datetime.datetime.now()
    mock_client = MongoClient()

    def teardown():
        print('\n')
        logger.info("Deleting database")
        mock_client.drop_database(DATABASE)
        logger.info("Database deleted")
        logger.info("Time to run test:{}".format(datetime.datetime.now() - start_time))

    request.addfinalizer(teardown)

    return mock_client


@pytest.fixture(scope='function')
def real_pymongo_client(request):
    """Get a client to the mongo database"""

    logger.info("Get a real pymongo client")
    start_time = datetime.datetime.now()
    mongo_client = pymongo.MongoClient()

    def teardown():
        print('\n')
        logger.info("Deleting database")
        mongo_client.drop_database(REAL_DATABASE)
        logger.info("Database deleted")
        logger.info("Time to run test:{}".format(datetime.datetime.now() - start_time))

    request.addfinalizer(teardown)

    return mongo_client


@pytest.fixture(scope='function')
def real_adapter(request, real_pymongo_client):
    """Get an adapter connected to mongo database"""
    logger.info("Connecting to database...")
    mongo_client = real_pymongo_client

    logger.info("Connecting to database %s", REAL_DATABASE)

    database = mongo_client[REAL_DATABASE]
    mongo_adapter = PymongoAdapter(database)

    mongo_adapter.load_indexes()

    logger.info("Connected to database")

    return mongo_adapter


@pytest.fixture(scope='function')
def adapter(request, pymongo_client):
    """Get an adapter connected to mongo database"""
    logger.info("Connecting to database...")
    mongo_client = pymongo_client

    database = mongo_client[DATABASE]
    mongo_adapter = PymongoAdapter(database)

    logger.info("Connected to database")

    return mongo_adapter


@pytest.fixture(scope='function')
def clinvar_database(request, adapter, clinvar_variant, user_obj, institute_obj, case_obj):
    "Returns an adapter to a database populated with one variant"

    user_id = user_obj['_id']
    institute_id = institute_obj['internal_id']
    case_id = case_obj['_id']

    adapter.add_clinvar_submission([clinvar_variant], user_id, institute_id, case_id)

    return adapter


@pytest.fixture(scope='function')
def institute_database(request, adapter, institute_obj, user_obj):
    "Returns an adapter to a database populated with institute"
    adapter.add_institute(institute_obj)
    adapter.add_user(user_obj)

    return adapter


@pytest.fixture(scope='function')
def real_institute_database(request, real_adapter, institute_obj, user_obj):
    "Returns an adapter to a database populated with institute"
    adapter = real_adapter
    adapter.add_institute(institute_obj)
    adapter.add_user(user_obj)

    return adapter


@pytest.fixture(scope='function')
def gene_database(request, institute_database, genes):
    "Returns an adapter to a database populated with user, institute, case and genes"
    adapter = institute_database

    gene_objs = load_hgnc_genes(
        adapter=adapter,
        genes=genes,
        build='37'
    )

    logger.info("Creating index on hgnc collection")
    adapter.hgnc_collection.create_index([('build', pymongo.ASCENDING),
                                          ('hgnc_symbol', pymongo.ASCENDING)])

    transcripts_handle = get_file_handle(transcripts37_reduced_path)
    load_transcripts(adapter, transcripts_handle, build='37')

    adapter.transcript_collection.create_index([('build', pymongo.ASCENDING),
                                                ('hgnc_id', pymongo.ASCENDING)])

    logger.info("Index done")

    return adapter


@pytest.fixture(scope='function')
def real_gene_database(request, real_institute_database, genes37_handle, hgnc_handle, exac_handle,
                  mim2gene_handle, genemap_handle, hpo_genes_handle):
    "Returns an adapter to a database populated with user, institute, case and genes"
    adapter = real_institute_database

    load_hgnc_genes(
        adapter=adapter,
        ensembl_lines=genes37_handle,
        hgnc_lines=hgnc_handle,
        exac_lines=exac_handle,
        mim2gene_lines=mim2gene_handle,
        genemap_lines=genemap_handle,
        hpo_lines=hpo_genes_handle,
        build='37'

    )

    logger.info("Creating index on hgnc collection")
    adapter.hgnc_collection.create_index([('build', pymongo.ASCENDING),
                                          ('hgnc_symbol', pymongo.ASCENDING)])
    logger.info("Index done")

    return adapter


@pytest.fixture(scope='function')
def hpo_database(request, gene_database, hpo_terms_handle, hpo_to_genes_handle, hpo_disease_handle):
    "Returns an adapter to a database populated with hpo terms"
    adapter = gene_database


    load_hpo(
        adapter=gene_database,
        hpo_lines=get_file_handle(hpoterms_reduced_path),
        hpo_gene_lines=get_file_handle(hpo_to_genes_reduced_path),
        disease_lines=get_file_handle(genemap2_reduced_path),
        hpo_disease_lines=get_file_handle(hpo_phenotype_to_terms_reduced_path),
    )

    return adapter


@pytest.fixture(scope='function')
def real_hpo_database(request, real_gene_database, hpo_terms_handle, hpo_to_genes_handle,
                      genemap_handle, hpo_disease_handle):
    "Returns an adapter to a database populated with hpo terms"
    adapter = real_gene_database

    load_hpo(
        adapter=gene_database,
        hpo_lines=hpo_terms_handle,
        hpo_gene_lines=hpo_to_genes_handle,
        disease_lines=genemap_handle,
        hpo_disease_lines=hpo_disease_handle,
    )

    return adapter


@pytest.fixture(scope='function')
def panel_database(request, gene_database, parsed_panel):
    "Returns an adapter to a database populated with user, institute and case"
    adapter = gene_database
    logger.info("Adding panel to adapter")

    adapter.load_panel(
        parsed_panel=parsed_panel,
    )

    return adapter


@pytest.fixture(scope='function')
def real_panel_database(request, real_gene_database, parsed_panel):
    "Returns an adapter to a database populated with user, institute and case"
    adapter = real_gene_database
    logger.info("Adding panel to real adapter")

    adapter.load_panel(
        parsed_panel=parsed_panel,
    )

    return adapter


@pytest.fixture(scope='function')
def case_database(request, panel_database, parsed_case):
    "Returns an adapter to a database populated with institute, user and case"
    adapter = panel_database
    case_obj = build_case(parsed_case, adapter)
    adapter._add_case(case_obj)

    return adapter


@pytest.fixture(scope='function')
def populated_database(request, panel_database, parsed_case):
    "Returns an adapter to a database populated with user, institute case, genes, panels"
    adapter = panel_database

    logger.info("Adding case to adapter")
    case_obj = build_case(parsed_case, adapter)
    adapter._add_case(case_obj)
    return adapter


@pytest.fixture(scope='function')
def real_populated_database(request, real_panel_database, parsed_case):
    "Returns an adapter to a database populated with user, institute case, genes, panels"
    adapter = real_panel_database

    logger.info("Adding case to real adapter")
    case_obj = build_case(parsed_case, adapter)
    adapter._add_case(case_obj)

    return adapter

@pytest.fixture(scope='function')
def variant_database(request, populated_database):
    """Returns an adapter to a database populated with user, institute, case
       and variants"""
    adapter = populated_database
    # Load variants
    case_obj = adapter.case_collection.find_one()

    adapter.load_variants(
        case_obj,
        variant_type='clinical',
        category='snv',
        rank_threshold=-10,
        build='37'
    )

    return adapter


@pytest.fixture(scope='function')
def real_variant_database(request, real_populated_database):
    """Returns an adapter to a database populated with user, institute, case
       and variants"""
    adapter = real_populated_database

    case_obj = adapter.case_collection.find_one()
    # Load variants
    adapter.load_variants(
        case_obj,
        variant_type='clinical',
        category='snv',
        rank_threshold=-10,
        build='37'
    )

    return adapter


@pytest.fixture(scope='function')
def sv_database(request, populated_database, variant_objs, sv_variant_objs):
    """Returns an adapter to a database populated with user, institute, case
       and variants"""
    adapter = populated_database

    case_obj = adapter.case_collection.find_one()
    # Load sv variants
    adapter.load_variants(
        case_obj,
        variant_type='clinical',
        category='sv',
        rank_threshold=-10,
        build='37'
    )

    return adapter


#############################################################
##################### Panel fixtures #####################
#############################################################
@pytest.fixture(scope='function')
def panel_info(request):
    "Return one panel info as specified in tests/fixtures/config1.ini"
    panel = {
        'date': datetime.datetime.now(),
        'file': panel_path,
        'type': 'clinical',
        'institute': 'cust000',
        'version': '1.0',
        'panel_name': 'panel1',
        'full_name': 'Test panel'
    }
    return panel


@pytest.fixture(scope='function')
def parsed_panel(request, panel_info):
    """docstring for parsed_panels"""
    panel = parse_gene_panel(
        path=panel_info['file'],
        institute=panel_info['institute'],
        panel_id=panel_info['panel_name'],
        panel_type=panel_info['type'],
        date=panel_info['date'],
        version=panel_info['version'],
        display_name=panel_info['full_name']
    )

    return panel


@pytest.fixture(scope='function')
def panel_obj(request, parsed_panel, gene_database):
    """docstring for parsed_panels"""
    panel = build_panel(parsed_panel, gene_database)

    return panel


@pytest.fixture(scope='function')
def gene_panels(request, parsed_case):
    """Return a list with the gene panels of parsed case"""
    panels = parsed_case['gene_panels']

    return panels


@pytest.fixture(scope='function')
def default_panels(request, parsed_case):
    """Return a list with the gene panels of parsed case"""
    panels = parsed_case['default_panels']

    return panels


#############################################################
##################### Variant fixtures #####################
#############################################################
@pytest.fixture(scope='function')
def basic_variant_dict(request):
    """Return a variant dict with the required information"""
    variant = {
        'CHROM': '1',
        'ID': '.',
        'POS': '10',
        'REF': 'A',
        'ALT': 'C',
        'QUAL': '100',
        'FILTER': 'PASS',
        'FORMAT': 'GT',
        'INFO': '.',
        'info_dict': {},
    }
    return variant


@pytest.fixture(scope='function')
def one_variant(request, variant_clinical_file):
    logger.info("Return one parsed variant")
    variant_parser = VCF(variant_clinical_file)

    for variant in variant_parser:
        break

    return variant


@pytest.fixture(scope='function')
def one_sv_variant(request, sv_clinical_file):
    logger.info("Return one parsed SV variant")
    variant_parser = VCF(sv_clinical_file)

    for variant in variant_parser:
        break

    return variant


@pytest.fixture(scope='function')
def rank_results_header(request, variant_clinical_file):
    logger.info("Return a VCF parser with one variant")
    variants = VCF(variant_clinical_file)
    rank_results = parse_rank_results_header(variants)

    return rank_results


@pytest.fixture(scope='function')
def sv_variants(request, sv_clinical_file):
    logger.info("Return a VCF parser many svs")
    variants = VCF(sv_clinical_file)
    return variants


@pytest.fixture(scope='function')
def variants(request, variant_clinical_file):
    logger.info("Return a VCF parser many svs")
    variants = VCF(variant_clinical_file)
    return variants


@pytest.fixture(scope='function')
def parsed_variant(request, one_variant, case_obj):
    """Return a parsed variant"""
    print('')
    variant_dict = parse_variant(one_variant, case_obj)
    return variant_dict


@pytest.fixture(scope='function')
def variant_obj(request, parsed_variant):
    """Return a variant object"""
    print('')
    institute_id = 'cust000'
    variant = build_variant(parsed_variant, institute_id=institute_id)
    return variant


@pytest.fixture(scope='function')
def cyvcf2_variant():
    """Return a variant object"""
    print('')

    class Cyvcf2Variant(object):
        def __init__(self):
            self.CHROM = '1'
            self.REF = 'A'
            self.ALT = ['C']
            self.POS = 10
            self.end = 11
            self.FILTER = None
            self.ID = '.'
            self.QUAL = None
            self.var_type = 'snp'
            self.INFO = {'RankScore': "123:10"}

    variant = Cyvcf2Variant()
    return variant


# @pytest.fixture(scope='function')
# def parsed_variant():
#     """Return variant information for a parsed variant with minimal information"""
#     variant = {'alternative': 'C',
#                'callers': {
#                    'freebayes': None,
#                    'gatk': None,
#                    'samtools': None
#                },
#                'case_id': 'cust000-643594',
#                'category': 'snv',
#                'chromosome': '2',
#                'clnsig': [],
#                'compounds': [],
#                'conservation': {'gerp': [], 'phast': [], 'phylop': []},
#                'dbsnp_id': None,
#                'end': 176968945,
#                'filters': ['PASS'],
#                'frequencies': {
#                    'exac': None,
#                    'exac_max': None,
#                    'thousand_g': None,
#                    'thousand_g_left': None,
#                    'thousand_g_max': None,
#                    'thousand_g_right': None},
#                'genes': [],
#                'genetic_models': [],
#                'hgnc_ids': [],
#                'ids': {'display_name': '1_10_A_C_clinical',
#                        'document_id': 'a1f1d2ac588dae7883f474d41cfb34b8',
#                        'simple_id': '1_10_A_C',
#                        'variant_id': 'e8e33544a4745f8f5a09c5dea3b0dbe4'},
#                'length': 1,
#                'local_obs_hom_old': None,
#                'local_obs_old': None,
#                'mate_id': None,
#                'position': 176968944,
#                'quality': 10.0,
#                'rank_score': 0.0,
#                'reference': 'A',
#                'samples': [{'alt_depth': -1,
#                             'display_name': 'NA12882',
#                             'genotype_call': None,
#                             'genotype_quality': None,
#                             'individual_id': 'ADM1059A2',
#                             'read_depth': None,
#                             'ref_depth': -1},
#                            {'alt_depth': -1,
#                             'display_name': 'NA12877',
#                             'genotype_call': None,
#                             'genotype_quality': None,
#                             'individual_id': 'ADM1059A1',
#                             'read_depth': None,
#                             'ref_depth': -1},
#                            {'alt_depth': -1,
#                             'display_name': 'NA12878',
#                             'genotype_call': None,
#                             'genotype_quality': None,
#                             'individual_id': 'ADM1059A3',
#                             'read_depth': None,
#                             'ref_depth': -1}],
#                'sub_category': 'snv',
#                'variant_type': 'clinical'}
#     return variant


@pytest.fixture(scope='function')
def parsed_sv_variant(request, one_sv_variant, case_obj):
    """Return a parsed variant"""
    print('')
    variant_dict = parse_variant(one_sv_variant, case_obj)
    return variant_dict


@pytest.fixture(scope='function')
def parsed_variants(request, variants, case_obj):
    """Get a generator with parsed variants"""
    print('')
    individual_positions = {}
    for i, ind in enumerate(variants.samples):
        individual_positions[ind] = i

    return (parse_variant(variant, case_obj,
                          individual_positions=individual_positions)
            for variant in variants)


@pytest.fixture(scope='function')
def parsed_sv_variants(request, sv_variants, case_obj):
    """Get a generator with parsed variants"""
    print('')
    individual_positions = {}
    for i, ind in enumerate(sv_variants.samples):
        individual_positions[ind] = i

    return (parse_variant(variant, case_obj,
                          individual_positions=individual_positions)
            for variant in sv_variants)


@pytest.fixture(scope='function')
def variant_objs(request, parsed_variants, institute_obj):
    """Get a generator with parsed variants"""
    print('')
    return (build_variant(variant, institute_obj)
            for variant in parsed_variants)


@pytest.fixture(scope='function')
def sv_variant_objs(request, parsed_sv_variants, institute_obj):
    """Get a generator with parsed variants"""
    print('')
    return (build_variant(variant, institute_obj)
            for variant in parsed_sv_variants)


#############################################################
##################### File fixtures #####################
#############################################################

@pytest.fixture
def config_file(request):
    """Get the path to a config file"""
    print('')
    return load_path


@pytest.fixture
def panel_1_file(request):
    """Get the path to a config file"""
    print('')
    return panel_path


@pytest.fixture
def hgnc_file(request):
    """Get the path to a hgnc file"""
    print('')
    return hgnc_reduced_path


@pytest.fixture
def transcripts_file(request):
    """Get the path to a ensembl transcripts file"""
    print('')
    return transcripts37_reduced_path


@pytest.fixture
def genes37_file(request):
    """Get the path to a ensembl genes file"""
    print('')
    return genes37_reduced_path

@pytest.fixture
def exac_file(request):
    """Get the path to a exac genes file"""
    print('')
    return exac_reduced_path


@pytest.fixture
def hpo_genes_file(request):
    """Get the path to the hpo genes file"""
    print('')
    return hpogenes_reduced_path


@pytest.fixture
def hpo_to_genes_file(request):
    """Get the path to the hpo to genes file"""
    print('')
    return hpo_to_genes_reduced_path


@pytest.fixture
def hpo_terms_file(request):
    """Get the path to the hpo terms file"""
    print('')
    return hpoterms_reduced_path


@pytest.fixture
def hpo_disease_file(request):
    """Get the path to the hpo disease file"""
    print('')
    return hpo_phenotype_to_terms_reduced_path


@pytest.fixture
def mim2gene_file(request):
    """Get the path to the mim2genes file"""
    print('')
    return mim2gene_reduced_path


@pytest.fixture
def genemap_file(request):
    """Get the path to the mim2genes file"""
    print('')
    return genemap2_reduced_path


@pytest.fixture(scope='function')
def variant_clinical_file(request):
    """Get the path to a variant file"""
    print('')
    return clinical_snv_path


@pytest.fixture(scope='function')
def sv_clinical_file(request):
    """Get the path to a variant file"""
    print('')
    return clinical_sv_path


@pytest.fixture(scope='function')
def ped_file(request):
    """Get the path to a ped file"""
    print('')
    return ped_path


@pytest.fixture(scope='function')
def scout_config(request, config_file):
    """Return a dictionary with scout configs"""
    print('')
    in_handle = get_file_handle(config_file)
    data = yaml.load(in_handle)
    return data


@pytest.fixture(scope='function')
def minimal_config(request, scout_config):
    """Return a minimal config"""
    config = scout_config
    config.pop('madeline')
    config.pop('vcf_sv')
    config.pop('vcf_snv_research')
    config.pop('vcf_sv_research')
    config.pop('gene_panels')
    config.pop('default_gene_panels')
    config.pop('rank_model_version')
    config.pop('rank_score_threshold')
    config.pop('human_genome_build')

    return config


@pytest.fixture
def panel_handle(request, panel_1_file):
    """Get a file handle to a gene panel file"""
    print('')
    return get_file_handle(panel_1_file)


@pytest.fixture
def hgnc_handle(request, hgnc_file):
    """Get a file handle to a hgnc file"""
    print('')
    return get_file_handle(hgnc_file)


@pytest.fixture
def hgnc_genes(request, hgnc_handle):
    """Get a dictionary with hgnc genes"""
    print('')
    return parse_hgnc_genes(hgnc_handle)


@pytest.fixture
def genes37_handle(request, genes37_file):
    """Get a file handle to a ensembl gene file"""
    print('')
    return get_file_handle(genes37_file)

@pytest.fixture
def transcripts_handle(request, transcripts_file):
    """Get a file handle to a ensembl transcripts file"""
    print('')
    return get_file_handle(transcripts_file)


@pytest.fixture
def transcripts(request, transcripts_handle):
    """Get the parsed ensembl transcripts"""
    print('')
    return parse_ensembl_transcripts(transcripts_handle)

@pytest.fixture
def parsed_transcripts(request, transcripts_handle, ensembl_genes):
    """Get the parsed ensembl transcripts"""
    print('')
    transcripts = parse_transcripts(transcripts_handle)
    for tx_id in transcripts:
        tx_info = transcripts[tx_id]
        ens_gene_id = tx_info['ensembl_gene_id']
        gene_obj = ensembl_genes.get(ens_gene_id)
        if not gene_obj:
            continue
        tx_info['hgnc_id'] = gene_obj['hgnc_id']
        tx_info['primary_transcripts'] = set(gene_obj.get('primary_transcripts', []))


    return transcripts


@pytest.fixture
def exac_handle(request, exac_file):
    """Get a file handle to a ensembl gene file"""
    print('')
    return get_file_handle(exac_file)


@pytest.fixture
def exac_genes(request, exac_handle):
    """Get the parsed exac genes"""
    print('')
    return parse_exac_genes(exac_handle)


@pytest.fixture
def hpo_genes_handle(request, hpo_genes_file):
    """Get a file handle to a hpo gene file"""
    print('')
    return get_file_handle(hpo_genes_file)


@pytest.fixture
def hpo_to_genes_handle(request, hpo_to_genes_file):
    """Get a file handle to a hpo to gene file"""
    print('')
    return get_file_handle(hpo_to_genes_file)


@pytest.fixture
def hpo_disease_handle(request, hpo_disease_file):
    """Get a file handle to a hpo disease file"""
    print('')
    return get_file_handle(hpo_disease_file)


@pytest.fixture
def mim2gene_handle(request, mim2gene_file):
    """Get a file handle to a mim2genes file"""
    print('')
    return get_file_handle(mim2gene_file)


@pytest.fixture
def genemap_handle(request, genemap_file):
    """Get a file handle to a mim2genes file"""
    print('')
    return get_file_handle(genemap_file)


@pytest.fixture
def hpo_genes(request, hpo_genes_handle):
    """Get the exac genes"""
    print('')
    return parse_hpo_genes(hpo_genes_handle)



# -*- coding: utf-8 -*-
"""

"main concept of MongoDB is embed whenever possible"
Ref: http://stackoverflow.com/questions/4655610#comment5129510_4656431
"""

from scout.constants import (CONSERVATION, GENETIC_MODELS, VARIANT_CALL, CLINSIG_MAP)

variant = dict(
    # document_id is a md5 string created by institute_genelist_caseid_variantid:
    _id = str, # required, same as document_id
    document_id = str, # required
    # variant_id is a md5 string created by chrom_pos_ref_alt (simple_id)
    variant_id = str, # required
    # display name is variant_id (no md5)
    display_name = str, # required

    # chrom_pos_ref_alt
    simple_id = str,
    # The variant can be either research or clinical.
    # For research variants we display all the available information while
    # the clinical variants have limited annotation fields.
    variant_type = str, # required, choices=('research', 'clinical'))

    category = str, # choices=('sv', 'snv', 'str')
    sub_category = str, # choices=('snv', 'indel', 'del', 'ins', 'dup', 'inv', 'cnv', 'bnd', 'str')
    mate_id = str, # For SVs this identifies the other end

    case_id = str, # case_id is a string like owner_caseid
    chromosome = str, # required
    position = int, # required
    end = int, # required
    length = int, # required
    reference = str, # required
    alternative = str, # required

    rank_score = float, # required
    variant_rank = int, # required
    rank_score_results = list, # List if dictionaries

    institute = str, # institute_id, required

    sanger_ordered = bool,
    validation = str, # Sanger validation, choices=('True positive', 'False positive')

    quality = float,
    filters = list, # list of strings
    samples = list, # list of dictionaries that are <gt_calls>
    genetic_models = list, # list of strings choices=GENETIC_MODELS
    compounds = list, # sorted list of <compound> ordering='combined_score'

    genes = list, # list with <gene>
    dbsnp_id = str,

    # str variant
    str_ru = str
    str_repid = str
    str_ref = str

    # Gene ids:
    hgnc_ids = list, # list of hgnc ids (int)
    hgnc_symbols = list, # list of hgnc symbols (str)
    panels = list, # list of panel names that the variant ovelapps

    # Frequencies:
    thousand_genomes_frequency = float,
    thousand_genomes_frequency_left = float,
    thousand_genomes_frequency_right = float,
    exac_frequency = float,
    max_thousand_genomes_frequency = float,
    max_exac_frequency = float,
    local_frequency = float,
    local_obs_old = int,
    local_obs_hom_old = int,
    local_obs_total_old = int, # default=638
    # Predicted deleteriousness:
    cadd_score = float,
    clnsig = list, # list of <clinsig>
    spidex = float,

    missing_data = bool, # default False

    # Callers
    gatk = str, # choices=VARIANT_CALL, default='Not Used'
    samtools = str, # choices=VARIANT_CALL, default='Not Used'
    freebayes = str, # choices=VARIANT_CALL, default='Not Used'

    # Conservation:
    phast_conservation = list, # list of str, choices=CONSERVATION
    gerp_conservation = list, # list of str, choices=CONSERVATION
    phylop_conservation = list, # list of str, choices=CONSERVATION
    # Database options:
    gene_lists = list,
    manual_rank = int, # choices=[0, 1, 2, 3, 4, 5]
    dismiss_variant = list,

    acmg_evaluation = str, # choices=ACMG_TERMS
)

compound = dict(
    # This must be the document_id for this variant
    variant = str, # required=True
    # This is the variant id
    display_name = str, # required
    combined_score = float, # required
)

clinsig = dict(
    value = int,
    accession = str,
    revstat = str
)

gt_call = dict(
    sample_id = str,
    display_name = str,
    genotype_call = str,
    allele_depths = list, # int
    read_depth = int,
    genotype_quality = int,
)



from scout.parse.ensembl import (parse_ensembl_line, parse_ensembl_transcripts, 
                                 parse_ensembl_transcript_request, parse_transcripts)

from pandas import DataFrame

def test_parse_ensembl_line(transcript_info):
    """Test to parse a line of ensembl transcript"""
    ## GIVEN some transcript information and a header
    header = ['Chromosome/scaffold name', 'Gene stable ID', 'Transcript stable ID', 
              'Transcript start (bp)', 'Transcript end (bp)', 'RefSeq mRNA ID', 
              'RefSeq mRNA predicted ID', 'RefSeq ncRNA ID']
    
    transcript_line = "{0}\t{1}\t{2}\t{3}\t{4}\t{5}\t{6}\t{7}".format(
        transcript_info['chrom'], transcript_info['ens_gene_id'], 
        transcript_info['ens_transcript_id'], transcript_info['start'], transcript_info['end'], 
        transcript_info['refseq_mrna'], transcript_info['refseq_mrna_pred'], 
        transcript_info['refseq_ncrna']
    )
    
    ## WHEN parsing the transcript line
    parsed_transcript = parse_ensembl_line(header=header, line=transcript_line)
    
    ## THEN assert the parsed transcript is as expected
    assert parsed_transcript['chrom'] == transcript_info['chrom']
    assert parsed_transcript['ensembl_gene_id'] == transcript_info['ens_gene_id']
    assert parsed_transcript['ensembl_transcript_id'] == transcript_info['ens_transcript_id']
    assert parsed_transcript['transcript_start'] == transcript_info['start']
    assert parsed_transcript['transcript_end'] == transcript_info['end']
    assert parsed_transcript['refseq_mrna'] == transcript_info['refseq_mrna']
    assert 'refseq_mrna_predicted' not in parsed_transcript
    assert 'refseq_ncrna' not in parsed_transcript
    
def test_parse_ensembl_request(transcript_info):
    """Test to parse a small dataframe line of ensembl transcript"""
    ## GIVEN a small transcript data framse
    
    tx_data = {
        'chromosome_name': transcript_info['chrom'],
        'ensembl_gene_id': [transcript_info['ens_gene_id']],
        'ensembl_transcript_id': [transcript_info['ens_transcript_id']],
        'transcript_start': [transcript_info['start']],
        'transcript_end': [transcript_info['end']],
        'refseq_mrna': [transcript_info['refseq_mrna']],
        'refseq_mrna_predicted': [transcript_info['refseq_mrna_pred']],
        'refseq_ncrna': [transcript_info['refseq_ncrna']],
    }
    df = DataFrame(tx_data)
    
    parsed_transcripts = parse_ensembl_transcript_request(df)
    
    parsed_tx = next(parsed_transcripts)
    
    ## THEN assert the parsed transcript is as expected
    assert parsed_tx['chrom'] == transcript_info['chrom']
    assert parsed_tx['ensembl_gene_id'] == transcript_info['ens_gene_id']
    assert parsed_tx['ensembl_transcript_id'] == transcript_info['ens_transcript_id']
    assert parsed_tx['transcript_start'] == transcript_info['start']
    assert parsed_tx['transcript_end'] == transcript_info['end']
    assert parsed_tx['refseq_mrna'] == transcript_info['refseq_mrna']
    assert parsed_tx['refseq_mrna_predicted'] == transcript_info['refseq_mrna_pred']
    assert parsed_tx['refseq_ncrna'] == transcript_info['refseq_ncrna']


def test_parse_ensembl_transcripts(transcripts_handle):
    """Test to parse all ensembl transcripts"""
    transcripts = parse_ensembl_transcripts(transcripts_handle)
    
    for transcript in transcripts:
        assert transcript['ensembl_transcript_id']
        assert transcript['ensembl_gene_id']


def test_parse_transcripts_file(transcripts_handle):
    """Test to parse all ensembl transcripts"""
    transcripts = parse_transcripts(transcripts_handle)
    
    for transcript_id in transcripts:
        transcript = transcripts[transcript_id]
        assert transcript['ensembl_transcript_id']
        assert transcript['ensembl_gene_id']

def test_parse_transcripts_data_frame(transcripts_df):
    """Test to parse all ensembl transcripts from data frame"""
    ## GIVEN a data frame with transcript information
    transcripts = parse_transcripts(transcripts_df)
    
    ## WHEN parsing the transcripts
    i = 0
    for i,transcript_id in enumerate(transcripts):
        transcript = transcripts[transcript_id]
    ## THEN assert they all got the mandatory ids
        assert transcript['ensembl_transcript_id']
        assert transcript['ensembl_gene_id']
    assert i > 0


# -*- coding: utf-8 -*-
from flask import abort, Blueprint, request, jsonify, redirect, url_for

from scout.server.extensions import store
from scout.server.utils import templated, public_endpoint
from . import controllers

genes_bp = Blueprint('genes', __name__, template_folder='templates')


@genes_bp.route('/genes')
@templated('genes/genes.html')
def genes():
    """Render seach box for genes."""
    query = request.args.get('query', '')
    if '|' in query:
        hgnc_id = int(query.split(' | ', 1)[0])
        return redirect(url_for('.gene', hgnc_id=hgnc_id))
    gene_q = store.all_genes().limit(20)
    return dict(genes=gene_q)


@genes_bp.route('/genes/<int:hgnc_id>')
@genes_bp.route('/genes/<hgnc_symbol>')
@templated('genes/gene.html')
def gene(hgnc_id=None, hgnc_symbol=None):
    """Render information about a gene."""
    if hgnc_symbol:
        query = store.hgnc_genes(hgnc_symbol)
        if query.count() == 1:
            hgnc_id = query.first()['hgnc_id']
        else:
            return redirect(url_for('.genes', query=hgnc_symbol))
    try:
        genes = controllers.gene(store, hgnc_id)
    except ValueError as error:
        return abort(404)

    return genes


@genes_bp.route('/api/v1/genes')
@public_endpoint
def api_genes():
    """Return JSON data about genes."""
    query = request.args.get('query')
    json_out = controllers.genes_to_json(store, query)
    return jsonify(json_out)



import logging

from scout.models.hgnc_map import HgncGene

LOG = logging.getLogger(__name__)

def build_phenotype(phenotype_info):
    phenotype_obj = {}
    phenotype_obj['mim_number'] = phenotype_info['mim_number']
    phenotype_obj['description'] = phenotype_info['description']
    phenotype_obj['inheritance_models'] = list(phenotype_info.get('inheritance', set()))
    phenotype_obj['status'] = phenotype_info['status']
    
    return phenotype_obj

def build_hgnc_gene(gene_info, build='37'):
    """Build a hgnc_gene object

        Args:
            gene_info(dict): Gene information

        Returns:
            gene_obj(dict)
    
            {
                '_id': ObjectId(),
                # This is the hgnc id, required:
                'hgnc_id': int, 
                # The primary symbol, required 
                'hgnc_symbol': str,
                'ensembl_id': str, # required
                'build': str, # '37' or '38', defaults to '37', required
                
                'chromosome': str, # required
                'start': int, # required
                'end': int, # required
                
                'description': str, # Gene description
                'aliases': list(), # Gene symbol aliases, includes hgnc_symbol, str
                'entrez_id': int,
                'omim_id': int,
                'pli_score': float,
                'primary_transcripts': list(), # List of refseq transcripts (str)
                'ucsc_id': str,
                'uniprot_ids': list(), # List of str
                'vega_id': str,
                'transcripts': list(), # List of hgnc_transcript
                
                # Inheritance information
                'inheritance_models': list(), # List of model names
                'incomplete_penetrance': bool, # Acquired from HPO
                
                # Phenotype information
                'phenotypes': list(), # List of dictionaries with phenotype information
            }
    """
    try:
        hgnc_id = int(gene_info['hgnc_id'])
    except KeyError as err:
        raise KeyError("Gene has to have a hgnc_id")
    except ValueError as err:
        raise ValueError("hgnc_id has to be integer")
    
    try:
        hgnc_symbol = gene_info['hgnc_symbol']
    except KeyError as err:
        raise KeyError("Gene has to have a hgnc_symbol")

    try:
        ensembl_id = gene_info['ensembl_gene_id']
    except KeyError as err:
        raise KeyError("Gene has to have a ensembl_id")

    try:
        chromosome = gene_info['chromosome']
    except KeyError as err:
        raise KeyError("Gene has to have a chromosome")
    
    try:
        start = int(gene_info['start'])
    except KeyError as err:
        raise KeyError("Gene has to have a start position")
    except TypeError as err:
        raise TypeError("Gene start has to be a integer")

    try:
        end = int(gene_info['end'])
    except KeyError as err:
        raise KeyError("Gene has to have a end position")
    except TypeError as err:
        raise TypeError("Gene end has to be a integer")

    gene_obj = HgncGene(
        hgnc_id=hgnc_id, 
        hgnc_symbol=hgnc_symbol, 
        ensembl_id=ensembl_id, 
        chrom=chromosome, 
        start=start, 
        end=end,
        build=build,
    )
    
    if gene_info.get('description'):
        gene_obj['description'] = gene_info['description']
        # LOG.debug("Adding info %s", gene_info['description'])
    
    if gene_info.get('previous_symbols'):
        gene_obj['aliases'] = gene_info['previous_symbols']

    if gene_info.get('entrez_id'):
        gene_obj['entrez_id'] = int(gene_info['entrez_id'])
    
    if gene_info.get('omim_id'):
        gene_obj['omim_id'] = int(gene_info['omim_id'])

    if gene_info.get('pli_score'):
        gene_obj['pli_score'] = float(gene_info['pli_score'])
    
    if gene_info.get('ref_seq'):
        gene_obj['primary_transcripts'] = gene_info['ref_seq']
    
    if gene_info.get('ucsc_id'):
        gene_obj['ucsc_id'] = gene_info['ucsc_id']
    
    if gene_info.get('uniprot_ids'):
        gene_obj['uniprot_ids'] = gene_info['uniprot_ids']

    if gene_info.get('vega_id'):
        gene_obj['vega_id'] = gene_info['vega_id']
    
    if gene_info.get('incomplete_penetrance'):
        gene_obj['incomplete_penetrance'] = True

    if gene_info.get('inheritance_models'):
        gene_obj['inheritance_models'] = gene_info['inheritance_models']

    phenotype_objs = []
    for phenotype_info in gene_info.get('phenotypes', []):
        phenotype_objs.append(build_phenotype(phenotype_info))
    
    if phenotype_objs:
        gene_obj['phenotypes'] = phenotype_objs

    for key in list(gene_obj):
        if gene_obj[key] is None:
            gene_obj.pop(key)

    return gene_obj



import logging

import click

from scout.utils.date import get_date
from scout.update.panel import update_panel


LOG = logging.getLogger(__name__)


@click.command('panel', short_help='Update a panel')
@click.option('--panel', '-p',
              help="Specify what panel to update",
              required=True
              )
@click.option('--version',
              type=float,
              help="Specify the version of a panel. If no version the latest panel is chosen.",
              )
@click.option('--update-date', '-d',
              # There will be more roles in the future
              help="Update the date for a panel",
              )
@click.option('--update-version',
              type=float,
              help="Change the version of a panel",
              )
@click.pass_context
def panel(context, panel, version, update_date, update_version):
    """
    Update a panel in the database
    """
    adapter = context.obj['adapter']

    # Check that the panel exists
    panel_obj = adapter.gene_panel(panel, version=version)

    if not panel_obj:
        LOG.warning("Panel %s (version %s) could not be found" % (panel, version))
        context.abort()

    date_obj = None
    if update_date:
        try:
            date_obj = get_date(update_date)
        except Exception as err:
            LOG.warning(err)
            context.abort()

    update_panel(
        adapter,
        panel,
        panel_version=panel_obj['version'],
        new_version=update_version,
        new_date=date_obj
    )



import logging

logger = logging.getLogger(__name__)

from scout.constants import (SPIDEX_HUMAN)

class QueryHandler(object):

    def build_query(self, case_id, query=None, variant_ids=None, category='snv'):
        """Build a mongo query

        These are the different query options:
            {
                'genetic_models': list,
                'chrom': str,
                'thousand_genomes_frequency': float,
                'exac_frequency': float,
                'clingen_ngi': int,
                'cadd_score': float,
                'cadd_inclusive": boolean,
                'genetic_models': list(str),
                'hgnc_symbols': list,
                'region_annotations': list,
                'functional_annotations': list,
                'clinsig': list,
                'clinsig_confident_always_returned': boolean,
                'variant_type': str(('research', 'clinical')),
                'chrom': str,
                'start': int,
                'end': int,
                'svtype': list,
                'gene_panels': list(str),
                'mvl_tag": boolean,
            }

        Arguments:
            case_id(str)
            query(dict): A dictionary with querys for the database
            variant_ids(list(str)): A list of md5 variant ids

        Returns:
            mongo_query : A dictionary in the mongo query format

        """
        query = query or {}
        mongo_query = {}
        logger.debug("Building a mongo query for %s" % case_id)
        mongo_query['case_id'] = case_id
        logger.debug("Querying category %s" % category)
        mongo_query['category'] = category

        # We need to check if there is any query specified in the input query
        mongo_query['variant_type'] = query.get('variant_type', 'clinical')
        logger.debug("Set variant type to %s", mongo_query['variant_type'])

        # Requests to filter based on gene panels, hgnc_symbols or
        # coordinate ranges must always be honored. They are always added to
        # query as top level, implicit '$and'. When both hgnc_symbols and a 
        # panel is used, addition of this is delayed until after the rest of 
        # the query content is clear.
 
        gene_query = []

        if query.get('hgnc_symbols') and query.get('gene_panels'):
            gene_query.append({'hgnc_symbols': {'$in': query['hgnc_symbols']}})
            gene_query.append({'panels': {'$in': query['gene_panels']}})
            mongo_query['$or']=gene_query
        else:
            if query.get('hgnc_symbols'):
                hgnc_symbols = query['hgnc_symbols']
                mongo_query['hgnc_symbols'] = {'$in': hgnc_symbols}
                logger.debug("Adding hgnc_symbols: %s to query" %
                             ', '.join(hgnc_symbols))

            if query.get('gene_panels'):
                gene_panels = query['gene_panels']
                mongo_query['panels'] = {'$in': gene_panels}

        if query.get('chrom'):
            chromosome = query['chrom']
            mongo_query['chromosome'] = chromosome
            #Only check coordinates if there is a chromosome
            if (query.get('start') and query.get('end')):
                mongo_query['position'] = {
                                                '$lte': int(query['end'])
                                        }

                mongo_query['end'] = {
                                        '$gte': int(query['start'])
                                    }

        mongo_query_minor = []

        # A minor, excluding filter criteria will hide variants in general,
        # but can be overridden by an including, major filter criteria
        # such as a Pathogenic ClinSig.
        # If there are no major criteria given, all minor criteria are added as a
        # top level '$and' to the query.
        # If there is only one major criteria given without any minor, it will also be
        # added as a top level '$and'.
        # Otherwise, major criteria are added as a high level '$or' and all minor criteria
        # are joined together with them as a single lower level '$and'.

        mongo_query_minor = []

        if query.get('gnomad_frequency') is not None:
            gnomad = query.get('gnomad_frequency')
            if gnomad == '-1':
                mongo_query['gnomad_frequency'] = {
                                                                '$exists': False
                                                            }

            else:
                # Replace comma with dot
                mongo_query_minor.append(
                    {
                        '$or': [
                            {
                                'gnomad_frequency':
                                    {
                                        '$lt': float(gnomad)
                                    }
                                },
                            {
                                'gnomad_frequency':
                                    {
                                        '$exists': False
                                    }
                            }
                        ]
                    }
                )
            logger.debug("Adding gnomad_frequency to query")

        if query.get('local_obs') is not None:
            mongo_query_minor.append({
                '$or': [
                    {'local_obs_old': None},
                    {'local_obs_old': {'$lt': query['local_obs'] + 1}},
                ]
            })

        if query.get('clingen_ngi') is not None:
            mongo_query_minor.append({
                '$or': [
                    {'clingen_ngi': {'$exists': False}},
                    {'clingen_ngi': {'$lt': query['clingen_ngi'] + 1}},
                ]
            })

        if query.get('spidex_human'):
            # construct spidex query. Build the or part starting with empty SPIDEX values
            spidex_human = query['spidex_human']

            spidex_query_or_part = []
            if ( 'not_reported' in spidex_human):
                spidex_query_or_part.append({'spidex': {'$exists': False}})

            for spidex_level in SPIDEX_HUMAN:
                if ( spidex_level in spidex_human ):
                    spidex_query_or_part.append({'$or': [ 
                                {'$and': [{'spidex': {'$gt': SPIDEX_HUMAN[spidex_level]['neg'][0]}},
                                          {'spidex': {'$lt': SPIDEX_HUMAN[spidex_level]['neg'][1]}}]},
                                {'$and': [{'spidex': {'$gt': SPIDEX_HUMAN[spidex_level]['pos'][0]}},
                                          {'spidex': {'$lt': SPIDEX_HUMAN[spidex_level]['pos'][1]}} ]} ]})
            
            mongo_query_minor.append({'$or': spidex_query_or_part })

        if query.get('cadd_score') is not None:
            cadd = query['cadd_score']
            cadd_query = {'cadd_score': {'$gt': float(cadd)}}
            logger.debug("Adding cadd_score: %s to query", cadd)

            if query.get('cadd_inclusive') is True:
                cadd_query = {
                    '$or': [
                        cadd_query,
                        {'cadd_score': {'$exists': False}}
                        ]}
                logger.debug("Adding cadd inclusive to query")

            mongo_query_minor.append(cadd_query)

        if query.get('genetic_models'):
            models = query['genetic_models']
            mongo_query_minor.append({'genetic_models': {'$in': models}})

            logger.debug("Adding genetic_models: %s to query" %
                         ', '.join(models))

        if query.get('functional_annotations'):
            functional = query['functional_annotations']
            mongo_query_minor.append({'genes.functional_annotation': {'$in': functional}})

            logger.debug("Adding functional_annotations %s to query" %
                         ', '.join(functional))

        if query.get('region_annotations'):
            region = query['region_annotations']
            mongo_query_minor.append({'genes.region_annotation': {'$in': region}})

            logger.debug("Adding region_annotations %s to query" %
                         ', '.join(region))

        if query.get('size'):
            size = query['size']
            size_query = {'length': {'$gt': int(size)}}
            logger.debug("Adding length: %s to query" % size)

            if query.get('size_inclusive') == 'yes':
                size_query = {
                    '$or': [
                        size_query,
                        {'length': {'$exists': False}}

                    ]}
                logger.debug("Adding size inclusive to query.")

            mongo_query_minor.append(size_query)

        if query.get('svtype'):
            svtype = query['svtype']
            mongo_query_minor.append({'sub_category': {'$in': svtype}})
            logger.debug("Adding SV_type %s to query" %
                         ', '.join(svtype))

        if query.get('depth'):
            logger.debug("add depth filter")
            mongo_query_minor.append({
                'tumor.read_depth': {
                    '$gt': query.get('depth'),
                }
            })

        if query.get('alt_count'):
            logger.debug("add min alt count filter")
            mongo_query_minor.append({
                'tumor.alt_depth': {
                    '$gt': query.get('alt_count'),
                }
            })

        if query.get('control_frequency'):
            logger.debug("add minimum control frequency filter")
            mongo_query_minor.append({
                'normal.alt_freq': {
                    '$lt': float(query.get('control_frequency')),
                }
            })

        if query.get('mvl_tag'):
            logger.debug("add managed variant list filter")
            mongo_query_minor.append({
                'mvl_tag': {
                    '$exists': True,
                }
            })

        mongo_query_major = None

        # Given a request to always return confident clinical variants,
        # add the clnsig query as a major criteria, but only
        # trust clnsig entries with trusted revstat levels.

        if query.get('clinsig'):
            rank = [int(item) for item in query['clinsig']]

            if query.get('clinsig_confident_always_returned') == True:

                trusted_revision_level = ['mult', 'single', 'exp', 'guideline']

                mongo_query_major = { "clnsig":
                        {
                            '$elemMatch': { 'value':
                                            { '$in': rank },
                                            'revstat':
                                            { '$in': trusted_revision_level }
                                          }
                         }
                    }

            else:
                logger.debug("add CLINSIG filter for rank: %s" %
                             ', '.join(str(query['clinsig'])))
                if mongo_query_minor:
                    mongo_query_minor.append({'clnsig.value': {'$in': rank}})
                else:
                    # if this is the only minor critera, use implicit and.
                    mongo_query['clnsig.value'] = {'$in': rank}

        if mongo_query_minor and mongo_query_major:
            if gene_query:
                mongo_query['$and'] = [ {'$or': gene_query}, 
                                        {'$or': [ {'$and': mongo_query_minor}, 
                                                  mongo_query_major ]} ]
            else:
                mongo_query['$or'] = [ {'$and': mongo_query_minor}, 
                                       mongo_query_major ]
        elif mongo_query_minor:
            if gene_query:
                mongo_query['$and'] = [ {'$or': gene_query},
                                        {'$and': mongo_query_minor} ]
            else:
                mongo_query['$and'] = mongo_query_minor
        elif mongo_query_major:
            # Restructure if more than ClinVar clnsig in the major category
            mongo_query['clnsig'] = mongo_query_major['clnsig']
            if gene_query:
                mongo_query['$and'] = [{ '$or': gene_query }]
        elif gene_query:
                mongo_query['$and'] = [{ '$or': gene_query }]
 
        if variant_ids:
            mongo_query['variant_id'] = {'$in': variant_ids}

            logger.debug("Adding variant_ids %s to query" % ', '.join(variant_ids))


        logger.debug("mongo query: %s", mongo_query)

        return mongo_query



# -*- coding: utf-8 -*-
import logging

from flask import (Blueprint, render_template)
from flask_login import current_user

from scout.constants import PHENOTYPE_GROUPS
from scout.server.extensions import store
from scout.server.utils import user_institutes

log = logging.getLogger(__name__)

blueprint = Blueprint('overview', __name__, template_folder='templates')


@blueprint.route('/overview')
def institutes():
    """Display a list of all user institutes."""
    institute_objs = user_institutes(store, current_user)
    institutes = []
    for ins_obj in institute_objs:
        sanger_recipients = []
        for user_mail in ins_obj.get('sanger_recipients',[]):
            user_obj = store.user(user_mail)
            if not user_obj:
                continue
            sanger_recipients.append(user_obj['name'])
        institutes.append(
            {
                'display_name': ins_obj['display_name'],
                'internal_id': ins_obj['_id'],
                'coverage_cutoff': ins_obj.get('coverage_cutoff', 'None'),
                'sanger_recipients': sanger_recipients,
                'frequency_cutoff': ins_obj.get('frequency_cutoff', 'None'),
                'phenotype_groups': ins_obj.get('phenotype_groups', PHENOTYPE_GROUPS)
            }
        )

    data = dict(institutes=institutes)
    return render_template(
        'overview/institutes.html', **data)



#!/usr/bin/env python
# encoding: utf-8
"""
get_genes.py

Parse all information for genes and build mongo engine objects.

Created by Måns Magnusson on 2014-11-10.
Copyright (c) 2014 __MoonsoInc__. All rights reserved.

"""
import logging

from scout.constants import SO_TERMS

def parse_genes(transcripts):
    """Parse transcript information and get the gene information from there.
    
    Use hgnc_id as identifier for genes and ensembl transcript id to identify transcripts
    
    Args:
        transcripts(iterable(dict))

    Returns:
      genes (list(dict)): A list with dictionaries that represents genes
    
    """
    # Dictionary to group the transcripts by hgnc_id
    genes_to_transcripts = {}
    
    # List with all genes and there transcripts
    genes = []

    hgvs_identifier = None
    canonical_transcript = None
    exon = None
    # Group all transcripts by gene
    for transcript in transcripts:
        # Check what hgnc_id a transcript belongs to
        hgnc_id = transcript['hgnc_id']
        hgnc_symbol = transcript['hgnc_symbol']

        if (transcript['is_canonical'] and transcript.get('coding_sequence_name')):
            hgvs_identifier = transcript.get('coding_sequence_name')
            canonical_transcript = transcript['transcript_id']
            exon = transcript['exon']

        # If there is a identifier we group the transcripts under gene
        if hgnc_id:
            if hgnc_id in genes_to_transcripts:
                genes_to_transcripts[hgnc_id].append(transcript)
            else:
                genes_to_transcripts[hgnc_id] = [transcript]
        else:
            if hgnc_symbol:
                if hgnc_symbol in genes_to_transcripts:
                    genes_to_transcripts[hgnc_symbol].append(transcript)
                else:
                    genes_to_transcripts[hgnc_symbol] = [transcript]

    # We need to find out the most severe consequence in all transcripts
    # and save in what transcript we found it
    
    # Loop over all genes
    for gene_id in genes_to_transcripts:
        # Get the transcripts for a gene
        gene_transcripts = genes_to_transcripts[gene_id]
        # This will be a consequece from SO_TERMS
        most_severe_consequence = None
        # Set the most severe score to infinity
        most_severe_rank = float('inf')
        # The most_severe_transcript is a dict
        most_severe_transcript = None
        
        most_severe_region = None
        
        most_severe_sift = None
        most_severe_polyphen = None
        
        # Loop over all transcripts for a gene to check which is most severe
        for transcript in gene_transcripts:
            hgnc_id = transcript['hgnc_id']
            hgnc_symbol = transcript['hgnc_symbol']
            # Loop over the consequences for a transcript
            for consequence in transcript['functional_annotations']:
                # Get the rank based on SO_TERM
                # Lower rank is worse
                new_rank = SO_TERMS[consequence]['rank']
                
                if new_rank < most_severe_rank:
                    # If a worse consequence is found, update the parameters
                    most_severe_rank = new_rank
                    most_severe_consequence = consequence
                    most_severe_transcript = transcript
                    most_severe_sift = transcript['sift_prediction']
                    most_severe_polyphen = transcript['polyphen_prediction']
                    most_severe_region = SO_TERMS[consequence]['region']

        gene = {
            'transcripts': gene_transcripts,
            'most_severe_transcript': most_severe_transcript,
            'most_severe_consequence': most_severe_consequence,
            'most_severe_sift': most_severe_sift,
            'most_severe_polyphen': most_severe_polyphen,
            'hgnc_id': hgnc_id,
            'hgnc_symbol': hgnc_symbol,
            'region_annotation': most_severe_region,
            'hgvs_identifier': transcript['coding_sequence_name'],
            'canonical_transcript': transcript['transcript_id'],
            'exon': transcript['exon'],
        }
        genes.append(gene)    

    return genes



import logging

import click

from scout.commands.utils import builds_option

from scout.export.transcript import export_transcripts

LOG = logging.getLogger(__name__)


@click.command('transcripts', short_help='Export transcripts')
@builds_option
@click.pass_context
def transcripts(context, build):
    """Export all transcripts to .bed like format"""
    LOG.info("Running scout export transcripts")
    adapter = context.obj['adapter']
    
    header = ["#Chrom\tStart\tEnd\tTranscript\tRefSeq\tHgncID"]

    for line in header:
        click.echo(line)

    transcript_string = ("{0}\t{1}\t{2}\t{3}\t{4}\t{5}")

    for tx_obj in export_transcripts(adapter):
        click.echo(transcript_string.format(
            tx_obj['chrom'],
            tx_obj['start'],
            tx_obj['end'],
            tx_obj['ensembl_transcript_id'],
            tx_obj.get('refseq_id',''),
            tx_obj['hgnc_id'],
            )
        )



from pprint import pprint as pp
from scout.load.transcript import load_transcripts

def test_load_transcripts(adapter, gene_bulk, transcripts_handle):
    # GIVEN a empty database
    assert adapter.all_genes().count() == 0
    assert adapter.transcripts().count() == 0
    
    # WHEN inserting a number of genes and some transcripts
    adapter.load_hgnc_bulk(gene_bulk)

    load_transcripts(adapter, transcripts_lines=transcripts_handle, build='37')

    # THEN assert all genes have been added to the database
    assert adapter.all_genes().count() == len(gene_bulk)
    
    # THEN assert that the transcripts where loaded loaded
    assert adapter.transcripts().count() > 0

def test_load_transcripts_request(adapter, gene_bulk, transcripts_df):
    # GIVEN a empty database
    assert adapter.all_genes().count() == 0
    assert adapter.transcripts().count() == 0
    
    # WHEN inserting a number of genes and some transcripts
    adapter.load_hgnc_bulk(gene_bulk)

    load_transcripts(adapter, transcripts_lines=transcripts_df, build='37')

    # THEN assert all genes have been added to the database
    assert adapter.all_genes().count() == len(gene_bulk)
    
    # THEN assert that the transcripts where loaded loaded
    assert adapter.transcripts().count() > 0



from pprint import pprint as pp

def test_update_caseid(real_oldcase_database, scout_config):
    # GIVEN a case with a "old-style" case id
    adapter = real_oldcase_database['adapter']
    old_case = real_oldcase_database['case']
    old_caseid = old_case['_id']
    ## THEN assert that the case has the old style id
    assert old_caseid == '-'.join([old_case['owner'], old_case['display_name']])
    ## THEN assert that the case has old id as case_id
    assert isinstance(adapter.case(old_caseid), dict)
    ## THEN assert that grabbing the case with disaply name does not work
    assert adapter.case(scout_config['family']) is None

    case_lists = {'suspects': None, 'causatives': None}
    for case_variants in case_lists:
        case_lists[case_variants] = old_case[case_variants][0]
        assert isinstance(adapter.variant(case_lists[case_variants]), dict)

    old_variant = real_oldcase_database['variant']
    old_evaluation = adapter.get_evaluations(old_variant)[0]
    assert old_evaluation['case_id'] == old_caseid
    assert old_evaluation['variant_specific'] == old_variant['_id']

    institute_obj = adapter.institute(old_case['owner'])
    events = adapter.events(institute_obj, case=old_case)
    for event_obj in events:
        assert event_obj['case'] == old_caseid

    # WHEN updating the case it as part of a upload
    adapter.load_case(scout_config, update=True)

    # THEN it should replace the case in the database
    assert adapter.case(old_caseid) is None
    new_case = adapter.case(scout_config['family'])
    assert isinstance(new_case, dict)
    # AND the suspect/causative variant should have an updated id
    for case_variants, old_variantid in case_lists.items():
        new_variantid = new_case[case_variants][0]
        assert new_variantid != old_variantid
        new_variant = adapter.variant_collection.find_one({'_id': new_variantid})
        assert isinstance(new_variant, dict)
        # AND the ACMG classification should have new case + variant ids
        new_evaluations = adapter.get_evaluations(new_variant)
        for new_evaluation in new_evaluations:
            assert new_evaluation['case_id'] == new_case['_id']
            assert new_evaluation['variant_specific'] == new_variant['_id']
    # AND update ids for events
    assert adapter.events(institute_obj, case=old_case).count() == 0
    new_events = adapter.events(institute_obj, case=new_case)
    for event_obj in new_events:
        assert event_obj['case'] == new_case['_id']




def build_clnsig(clnsig_info):
    """docstring for build_clnsig"""
    clnsig_obj = dict(
        value = clnsig_info['value'],
        accession = clnsig_info.get('accession'),
        revstat = clnsig_info.get('revstat')
    )
    
    return clnsig_obj


from scout.parse.hpo import (parse_hpo_phenotype, parse_hpo_phenotypes, parse_hpo_obo)

def test_parse_hpo_line():
    ## GIVEN a line that describes a hpo term
    hpo_line = "HP:0000878\t11 pairs of ribs\t126792\tB3GALT6"
    ## WHEN parsing the line
    hpo_info = parse_hpo_phenotype(hpo_line)
    ## THEN assert that the correct information was parsed
    assert hpo_info['hpo_id'] == "HP:0000878"
    assert hpo_info['description'] == "11 pairs of ribs"
    assert hpo_info['hgnc_symbol'] == "B3GALT6"

def test_parse_hpo_lines():
    ## GIVEN some lines that correspends to a hpo file
    hpo_lines = [
        "#Format: HPO-ID<tab>HPO-Name<tab>Gene-ID<tab>Gene-Name",
        "HP:0000878\t11 pairs of ribs\t126792\tB3GALT6",
        "HP:0000878\t11 pairs of ribs\t5932\tRBBP8"
    ]
    ## WHEN parsing the lines
    hpo_dict = parse_hpo_phenotypes(hpo_lines)
    ## THEN assert that the correct information was parsed
    assert len(hpo_dict) == 1
    assert "HP:0000878" in hpo_dict
    assert set(hpo_dict['HP:0000878']['hgnc_symbols']) == set(['B3GALT6', 'RBBP8'])

def test_parse_hpo_obo():
    hpo_info = [
        "[Term]",
        "id: HP:0000003",
        "name: Multicystic kidney dysplasia",
        "alt_id: HP:0004715",
        "def: Multicystic dysplasia of the kidney is characterized by multiple cysts of varying size in the kidney and the absence of a normal pelvicaliceal system. The condition is associated with ureteral or ureteropelvic atresia, and the affected kidney is nonfunctional. [HPO:curators]",
        "comment: Multicystic kidney dysplasia is the result of abnormal fetal renal development in which the affected kidney is replaced by multiple cysts and has little or no residual function. The vast majority of multicystic kidneys are unilateral. Multicystic kidney can be diagnosed on prenatal ultrasound.",
        'synonym: "Multicystic dysplastic kidney" EXACT []',
        'synonym: "Multicystic kidneys" EXACT []',
        'synonym: "Multicystic renal dysplasia" EXACT []',
        "xref: MSH:D021782",
        "xref: SNOMEDCT_US:204962002",
        "xref: SNOMEDCT_US:82525005",
        "xref: UMLS:C3714581",
        "is_a: HP:0000107 ! Renal cyst",
        
    ]
    hpo_terms = parse_hpo_obo(hpo_info)
    
    for hpo_term in hpo_terms:
        assert hpo_term['hpo_id'] == "HP:0000003"
        assert hpo_term['description'] == "Multicystic kidney dysplasia"

def test_parse_hpo_terms(hpo_terms_handle):
    hpo_terms = parse_hpo_obo(hpo_terms_handle)
    
    for hpo_term in hpo_terms:
        assert hpo_term['hpo_id']
